import matplotlib.pyplot as plt
import matplotlib as mpl
import meteva
import numpy as np
import  math
#from sklearn.linear_model import LinearRegression
from  matplotlib import  cm
from matplotlib.ticker import NullFormatter, FixedLocator
import copy
import pandas as pd
import datetime
import scipy.stats as st


def scatter_regress(ob, fo,member_list = None, rtype="linear",vmax = None,vmin = None, ncol = None,save_path=None,show = False,dpi = 300, title="散点回归图",
                    sup_fontsize = 10,width = None,height = None,ylabel ="观测"):
    '''
    绘制观测-预报散点图和线性回归曲线
    :param Ob: 实况数据  任意维numpy数组
    :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param save_path:图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    :return:图片，包含散点图和线性回归图,横坐标为观测值，纵坐标为预报值，横坐标很纵轴标取值范围自动设为一致，在图形中间添加了完美预报的参考线。
    '''

    num_max = max(np.max(ob), np.max(fo))
    num_min = min(np.min(ob), np.min(fo))
    dmm = num_max - num_min
    if (num_min < 0):
        num_min -= 0.1 * dmm
    else:
        num_min -= 0.1 * dmm
        if num_min < 0:  # 如果开始全大于，则最低值扩展不超过0
            num_min = 0
    num_max += dmm * 0.1
    if vmax is not None:
        num_max = vmax
    if vmin is not None:
        num_min = vmin
    dmm = num_max - num_min


    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    sub_plot_num = new_Fo_shape[0]

    if ncol is None:
        if sub_plot_num ==1:
            ncols = 1
        elif sub_plot_num %2 == 0:
            ncols = 2
        else:
            ncols = 3
    else:
        ncols = ncol

    nrows = math.ceil(new_Fo_shape[0] / ncols)

    if height is None:
        if nrows==1:
            if ncols <3:
                height_fig = 3.5
            else:
                height_fig = 2.5
        else:
            if ncols > nrows:
                height_fig = 6
            else:
                height_fig = 7
    else:
        height_fig = height

    height_suptitle = 0.4
    height_xticks_title = 0.1
    height_hspace = 0.6
    heidht_axis = (height_fig - height_suptitle - height_xticks_title - height_hspace * (nrows - 1)) / nrows
    width_axis = heidht_axis
    width_yticks = 0.1
    width_wspace = width_yticks * 5
    if width is None:
        width_fig = width_axis * ncols + width_wspace * (ncols - 1) + width_yticks
    else:
        width_fig = width

    grid_count = meteva.base.grid([num_min,num_max,(num_max-num_min)/100],[num_min,num_max,(num_max-num_min)/100])


    fig = plt.figure(figsize=(width_fig,height_fig),dpi = dpi)
    for line in range(new_Fo_shape[0]):
        ob = ob.flatten()
        fo = new_Fo[line,:].flatten()
        markersize = width_axis * heidht_axis / np.sqrt(ob.size)
        if markersize < 0.3:
            markersize = 0.3
        elif markersize > 20:
            markersize = 20

        plt.subplot(nrows, ncols, line + 1)
        df =pd.DataFrame({"lon":fo,"lat":ob})
        sta_xy = meteva.base.sta_data(df)
        sta_xy["time"] = datetime.datetime(2020,1,1,0)
        sta_xy["data0"] = 1
        grd_count = meteva.base.near.add_stavalue_to_nearest_grid(sta_xy,grid = grid_count)
        sta_count = meteva.base.interp_gs_linear(grd_count,sta_xy)
        colors = sta_count["data0"]
        sort_index = colors.argsort()
        fo_s = fo[sort_index]
        ob_s = ob[sort_index]
        colors = colors[sort_index]
        #plt.scatter(fo_s, ob_s, c=colors,s = markersize,cmap="tab20c")
        plt.scatter(fo_s, ob_s, c=colors, s=markersize, cmap="turbo")
        #plt.plot(fo, ob, '.', color='b', markersize=markersize)

        plt.subplots_adjust(left=0, bottom=0.0, right=1.0, top = 1 - height_suptitle/height_fig,
                            hspace=height_hspace/heidht_axis,wspace=width_wspace/width_axis)
        if rtype == "rate":
            ob_line = np.arange(num_min, num_max, dmm / 30)
            rate = np.mean(ob) / np.mean(fo)
            fo_rg = ob_line * np.mean(ob) / np.mean(fo)
            plt.plot(ob_line, fo_rg, color="k")
            rg_text2 = "Y = " + '%.2f' % rate + "* X"
            plt.text(num_min + 0.05 * dmm, num_min + 0.92 * dmm, rg_text2, fontsize=0.8 * sup_fontsize, color="r")
        elif rtype == "linear":
            #X = np.zeros((len(fo), 1))
            #X[:, 0] = fo
            #clf = LinearRegression().fit(X, ob)
            # 斜率，截距，r 值，p 值，标准误差
            slope, intercept, r_value, p_value, std_err = st.linregress(fo, ob)
            ob_line = np.arange(num_min, num_max, dmm / 30)
            # X = np.zeros((len(ob_line), 1))
            # X[:, 0] = ob_line
            #fo_rg = clf.predict(X)
            fo_rg = slope * ob_line + intercept
            plt.plot(ob_line, fo_rg, color="k")
            #rg_text2 = "Y = " + '%.2f' % (clf.coef_[0]) + "* X + " + '%.2f' % (clf.intercept_)
            if intercept >=0:
                rg_text2 = "Y = " + '%.2f' % (slope) + "* X + " + '%.2f' % (intercept)
            else:
                rg_text2 = "Y = " + '%.2f' % (slope) + "* X - " + '%.2f' % (-intercept)

            plt.text(num_min + 0.05 * dmm, num_min + 0.92 * dmm, rg_text2, fontsize=0.8 * sup_fontsize, color="r")
        corr1 = meteva.method.corr(ob,fo)
        re_text1 = "corr = "+ '%.2f' % (corr1)
        plt.text(num_min + 0.05 * dmm, num_min + 0.85 * dmm, re_text1, fontsize=0.8 * sup_fontsize, color="r")


        plt.plot(ob_line, ob_line, '--', color="k",linewidth = 0.5)
        plt.xlim(num_min, num_max)
        plt.ylim(num_min, num_max)
        plt.xticks(fontsize = 0.8 * sup_fontsize)
        plt.yticks(fontsize = 0.8 * sup_fontsize)


        if member_list is None:
            #plt.title('预报'+str(line+1),fontsize = 0.9 * fontsize_sup)
            plt.xlabel('预报'+str(line+1), fontsize=0.9 * sup_fontsize)
        else:
            #plt.title(member_list[line],fontsize = 0.9 * fontsize_sup)
            plt.xlabel(member_list[line], fontsize=0.9 * sup_fontsize)
        #plt.xlabel("预报", fontsize=0.9 * fontsize_sup)

        plt.ylabel(ylabel, fontsize=0.9 * sup_fontsize)
        plt.rcParams['xtick.direction'] = 'in'  # 将x轴的刻度线方向设置抄向内
        plt.rcParams['ytick.direction'] = 'in'  # 将y轴的刻度方知向设置向内
        #plt.grid(linestyle='--', linewidth=0.5)
    titlelines = title.split("\n")
    fig.suptitle(title, fontsize=sup_fontsize, y=0.99+0.01 * len(titlelines))

    if save_path is None:
        show = True
    else:
        meteva.base.tool.path_tools.creat_path(save_path)
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()
    return None




def pdf_plot(ob, fo,member_list = None,vmax = None,vmin = None, save_path=None,  show = False,dpi = 300,title="频率匹配检验图",
             sup_fontsize = 10,width = None,height = None,yscale = None,grid = False):
    '''
    sorted_ob_fo 将传入的两组数据先进行排序
    然后画出折线图
    ----------------
    :param Ob: 实况数据  任意维numpy数组
    :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param save_path: 图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    :return:图片，包含频率匹配映射关系图,横坐标为观测值，纵坐标为预报值，横坐标很纵轴标取值范围自动设为一致，在图形中间添加了完美预报的参考线。
    '''
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    if width is None:
        width = 8
    if height is None:
        height = width * 0.45

    fig = plt.figure(figsize=(width, height),dpi = dpi)
    num_max = max(np.max(ob), np.max(fo))
    num_min = min(np.min(ob), np.min(fo))
    dmm = num_max - num_min
    if (num_min != 0):
        num_min -= 0.1 * dmm
    num_max += dmm * 0.1
    if vmax is not None:
        num_max = vmax
    if vmin is not None:
        num_min = vmin
    dmm = num_max - num_min
    ob= ob.flatten()

    ob_sorted = np.sort(ob.flatten())

    ob_sorted_smooth = ob_sorted
    ob_sorted_smooth[1:-1] = 0.5 * ob_sorted[1:-1] + 0.25 * (ob_sorted[0:-2] + ob_sorted[2:])
    ax = plt.subplot(1, 2, 1)
    y = np.arange(len(ob_sorted_smooth)) / (len(ob_sorted_smooth))
    plt.plot(ob_sorted_smooth, y, label="观测")

    dss = ob_sorted_smooth - ob_sorted_smooth[0]
    fnq =[(dss != 0).argmax()]

    for line in range(new_Fo_shape[0]):
        if member_list is None:
            if new_Fo_shape[0] == 1:
                label = '预报'
            else:
                label = '预报' + str(line + 1)
        else:
            label = member_list[line]
        fo_sorted = np.sort(new_Fo[line, :].flatten())
        fo_sorted_smooth = fo_sorted
        fo_sorted_smooth[1:-1] = 0.5 * fo_sorted[1:-1] + 0.25 * (fo_sorted[0:-2] + fo_sorted[2:])
        plt.plot(fo_sorted_smooth, y, label=label)
        plt.xlabel("变量值", fontsize=0.9 * sup_fontsize)
        #plt.xlim(num_min, num_max)

        plt.ylabel("累积概率", fontsize=0.9 * sup_fontsize)
        plt.title("概率分布函数对比图", fontsize=0.9 * sup_fontsize)
        yticks = np.arange(0, 1.01, 0.1)
        plt.yticks(yticks, fontsize=0.8 * sup_fontsize)
        plt.xticks(fontsize=0.8 * sup_fontsize)
        plt.legend(loc="lower right")
        if yscale =="log":
            ax.set_yscale('log')
            major_locator = [0.00001,0.0001,0.001,0.01,0.1,0.5]
            ax.yaxis.set_minor_formatter(NullFormatter())
            ax.yaxis.set_major_locator(FixedLocator(major_locator))
            ax.set_yticklabels(major_locator)
        elif yscale == "logit":
            major_locator=[0.1,0.5,0.9,0.99,0.999,0.9999,0.99999]
            ax.set_yscale('logit')
            ax.yaxis.set_minor_formatter(NullFormatter())
            ax.yaxis.set_major_locator(FixedLocator(major_locator))
            ax.set_yticklabels(major_locator)
            dss = fo_sorted_smooth - fo_sorted_smooth[0]
            fnq.append((dss!=0).argmax())

        else:
            plt.ylim(0, 1)

    if yscale == "logit":
        minfnq = min(fnq)/ob_sorted_smooth.size
        plt.ylim(minfnq,1)


        #plt.yscale('logit')
    if grid:
        plt.grid()

    plt.subplot(1, 2, 2)
    ob_line = np.arange(num_min, num_max, dmm / 30)
    plt.plot(ob_line, ob_line, '--')
    for line in range(new_Fo_shape[0]):
        if member_list is None:
            if new_Fo_shape[0] == 1:
                label = '预报'
            else:
                label = '预报' + str(line + 1)
        else:
            label = member_list[line]
        fo_sorted = np.sort(new_Fo[line, :].flatten())
        fo_sorted_smooth = fo_sorted
        fo_sorted_smooth[1:-1] = 0.5 * fo_sorted[1:-1] + 0.25 * (fo_sorted[0:-2] + fo_sorted[2:])
        plt.plot(fo_sorted_smooth, ob_sorted_smooth, linewidth=2, label=label)
        plt.xlim(num_min, num_max)
        plt.ylim(num_min, num_max)
        plt.xlabel("预报", fontsize=0.9 * sup_fontsize)
        plt.ylabel("观测", fontsize=0.9 * sup_fontsize)
        plt.title("频率匹配映射关系图", fontsize=0.9 * sup_fontsize)
        plt.legend(loc="lower right", fontsize=0.9 * sup_fontsize)
        plt.yticks(fontsize=0.8 * sup_fontsize)
        plt.xticks(fontsize=0.8 * sup_fontsize)
    if title is not None:
        plt.suptitle(title + "\n", y=1.00, fontsize=sup_fontsize)
    if grid:
        plt.grid()
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()


def box_plot_continue(ob, fo,  member_list=None,vmax = None,vmin = None, save_path=None, show = False,dpi = 300,title="频率对比箱须图",
                      sup_fontsize = 10,width = None,height = None,):
    '''
    box_plot 画一两组数据的箱型图
    ---------------
    :param Ob: 实况数据  任意维numpy数组
    :param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param save_path: 图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    :return:图片，包含箱须图，等级包括,横坐标为"观测"、"预报"，纵坐标为数据值
    '''
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    list_fo = list(new_Fo)

    xticks = ['观测']
    if member_list is None:
        if new_Fo_shape[0] == 1:
            xticks.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                xticks.append('预报' + str(i + 1))
    else:
        xticks.extend(member_list)

    #print(width)
    new_list_fo = []
    ob = ob.flatten()
    new_list_fo.append(ob)
    for fo_piece in list_fo:
        new_list_fo.append(fo_piece.flatten())

    tuple_of_ob = tuple(new_list_fo)
    if width is None:
        width = meteva.base.plot_tools.caculate_axis_width(xticks, sup_fontsize) +0.5
        if width >10:
            for i in range(len(xticks)):
                if i % 2 ==1:
                    xticks[i] ="|\n" + xticks[i]
            width = 10
        elif width < 5:
            width = 5

    if height is None:
        height = width/2

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    markersize = 5 * width * height / np.sqrt(ob.size)
    if markersize < 1:
        markersize = 1
    elif markersize > 20:
        markersize = 20
    colors_list= []
    colors = cm.get_cmap('rainbow', 128)
    for i in range(len(xticks)):
        color_grade = i / len(xticks)
        colors_list.append(colors(color_grade))

    bplot = plt.boxplot(tuple_of_ob, showfliers=True, patch_artist=True, labels=xticks)


    plt.xticks(fontsize = 0.9 * sup_fontsize)
    plt.yticks(fontsize = 0.9 * sup_fontsize)

    plt.title(title,fontsize = sup_fontsize)
    for i, item in enumerate(bplot["boxes"]):
        item.set_facecolor(colors_list[i])
    #plt.title(title, fontsize=sup_fontsize)

    if vmin is not None or vmax is not None:
        if vmin is not None:
            if vmax is None:
                vmax = max(np.max(ob), np.max(fo))
                dmax = vmax - vmin
                plt.ylim(vmin,vmax+ dmax * 0.05)
            else:
                plt.ylim(vmin, vmax)
        else:
            vmin =  min(np.min(ob), np.min(fo))
            dmax = vmax - vmin
            plt.ylim(vmin- dmax * 0.05,vmax)


    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()



'''
def box_plot_continue(ob, fo,  member_list=None,vmax = None,vmin = None, save_path=None, show = False,dpi = 300,title="频率对比箱须图",
                      sup_fontsize = 10,width = None,height = None,):
    
    #box_plot 画一两组数据的箱型图
    #---------------
    #:param Ob: 实况数据  任意维numpy数组
    #:param Fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    #:param save_path: 图片保存路径，缺省时不输出图片，而是以默认绘图窗口形式展示
    #:return:图片，包含箱须图，等级包括,横坐标为"观测"、"预报"，纵坐标为数据值
    
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    list_fo = list(new_Fo)

    xticks = ['观测']
    if member_list is None:
        if new_Fo_shape[0] == 1:
            xticks.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                xticks.append('预报' + str(i + 1))
    else:
        xticks.extend(member_list)

    #print(width)
    new_list_fo = []
    ob = ob.flatten()
    new_list_fo.append(ob)
    for fo_piece in list_fo:
        new_list_fo.append(fo_piece.flatten())

    tuple_of_ob = tuple(new_list_fo)
    if width is None:
        width = meteva.base.plot_tools.caculate_axis_width(xticks, sup_fontsize) +0.5
        if width >10:
            for i in range(len(xticks)):
                if i % 2 ==1:
                    xticks[i] ="|\n" + xticks[i]
            width = 10
        elif width < 5:
            width = 5

    if height is None:
        height = width/2

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    markersize = 5 * width * height / np.sqrt(ob.size)
    if markersize < 1:
        markersize = 1
    elif markersize > 20:
        markersize = 20
    colors_list= []
    colors = cm.get_cmap('rainbow', 128)
    for i in range(len(xticks)):
        color_grade = i / len(xticks)
        colors_list.append(colors(color_grade))

    bplot = plt.boxplot(tuple_of_ob, showfliers=True, patch_artist=True, labels=xticks)


    plt.xticks(fontsize = 0.9 * sup_fontsize)
    plt.yticks(fontsize = 0.9 * sup_fontsize)

    plt.title(title,fontsize = sup_fontsize)
    for i, item in enumerate(bplot["boxes"]):
        item.set_facecolor(colors_list[i])
    #plt.title(title, fontsize=sup_fontsize)

    if vmin is not None or vmax is not None:
        if vmin is not None:
            if vmax is None:
                vmax = max(np.max(ob), np.max(fo))
                dmax = vmax - vmin
                plt.ylim(vmin,vmax+ dmax * 0.05)
            else:
                plt.ylim(vmin, vmax)
        else:
            vmin =  min(np.min(ob), np.min(fo))
            dmax = vmax - vmin
            plt.ylim(vmin- dmax * 0.05,vmax)


    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()

'''


def taylor_diagram(ob, fo,member_list=None, save_path=None,show = False,dpi = 300, title="",
                sup_fontsize =10,width = None,height = None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''

    leftw = 0.3
    rightw = 1.5
    uphight = 1.2
    lowhight = 1.2
    axis_size_x = 3
    axis_size_y = 3
    if width is None:
        width = axis_size_x + leftw + rightw

    if height is None:
        height = axis_size_y + uphight + lowhight

    stds = meteva.method.ob_fo_std(ob,fo)
    corrs = meteva.method.corr(ob,fo)
    corrs1 = [1]
    if isinstance(corrs,float):
        corrs1.append(corrs)
    else:
        for i in range(len(corrs)):
            corrs1.append(corrs[i])


    fig = plt.figure(figsize=(width, height),dpi=dpi)
    ax1 = fig.add_axes([leftw / width, lowhight / width, axis_size_x / width, axis_size_y / height])


    max_stds = max(stds)

    dif = (max_stds) / 10.0
    if dif == 0:
        inte = 1
    else:
        inte = math.pow(10, math.floor(math.log10(dif)))
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte

    if r < 2.3:
        inte = inte * 2
    elif r < 7:
        inte = inte * 5
    else:
        inte = inte * 10
    vmax = inte * ((int)(max_stds / inte) + 1)

    std_list = np.arange(inte,vmax+inte/2,inte)

    corr_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9,0.98]
    #画观测弧线
    # 画弧线
    angle = np.arange(0,math.pi/2,math.pi/1000)
    std = stds[0]
    x2 = np.cos(angle) * std
    y2 = np.sin(angle) * std
    plt.plot(x2, y2,color = "steelblue",linewidth = 1)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std
        y2 = np.sin(angle) * std
        if i <len(std_list)-1:
            plt.plot(x2, y2, ":", color="k", linewidth=0.5)
        else:
            plt.plot(x2, y2,color = "k",linewidth = 0.5)


    #画围绕观测的弧线
    angle = np.arange(0,math.pi,math.pi/1000)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std + stds[0]
        y2 = np.sin(angle) * std
        dis = np.sqrt(x2 * x2 + y2 * y2)
        x2 = x2[dis < vmax]
        y2 = y2[dis < vmax]

        plt.plot(x2, y2, ":", color="g", linewidth=0.5)

    #相关系数射线
    r0 = np.arange(0,vmax,inte/100)
    for i in range(len(corr_list)):
        corr = corr_list[i]
        angle = np.arccos(corr)
        x1 = r0 * np.cos(angle)
        y1 = r0 * np.sin(angle)
        ax1.plot(x1, y1, '-.', color='b', linewidth=0.4)
        rt = vmax* 1.01
        xt = rt * np.cos(angle)
        yt = rt * np.sin(angle)
        ax1.text(xt, yt, str(corr),fontsize = sup_fontsize * 0.8)


    angle = 60 * math.pi/180
    rt = vmax * 1.01
    xt = rt * np.cos(angle)
    yt = rt * np.sin(angle)

    ax1.text(xt, yt,"相关系数" , fontsize=sup_fontsize * 0.8,rotation=-30,)

    ax1.set_xticks(std_list)
    ax1.set_yticks(std_list)
    ax1.set_xlim(0,vmax)
    ax1.set_ylim(0, vmax)
    ax1.spines['right'].set_visible(False)
    ax1.spines['top'].set_visible(False)
    ax1.set_xlabel("标准差")
    ax1.set_ylabel("标准差")



    # 画预报观测点
    labels = ["观测"]
    labels.extend(member_list)
    for i in range(len(stds)):
        corr = corrs1[i]
        angle = np.arccos(corr)
        xp = stds[i] * np.cos(angle)
        yp = stds[i] * np.sin(angle)
        ax1.plot(xp, yp,"o", label=labels[i], markersize=6)
    lines, label1 = ax1.get_legend_handles_labels()

    if len(stds)> 7:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.4, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    elif len(stds)>5:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.2, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    else:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.1, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    ax1.add_artist(legend2)


    title = title + "\n"
    ax1.set_title(title,fontsize = sup_fontsize)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()


def taylor_diagram_(ob, fo,member_list=None, save_path=None,show = False,dpi = 300, title="",
                sup_fontsize =10,width = None,height = None):
    '''

    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''

    leftw = 0.3
    rightw = 1.5
    uphight = 1.2
    lowhight = 1.2
    axis_size_x = 3
    axis_size_y = 3
    if width is None:
        width = axis_size_x + leftw + rightw

    if height is None:
        height = axis_size_y + uphight + lowhight

    stds = meteva.method.ob_fo_std(ob,fo)
    corrs = meteva.method.corr(ob,fo)
    corrs1 = [1]
    if isinstance(corrs,float):
        corrs1.append(corrs)
    else:
        for i in range(len(corrs)):
            corrs1.append(corrs[i])


    fig = plt.figure(figsize=(width, height),dpi=dpi)
    ax1 = fig.add_axes([leftw / width, lowhight / width, axis_size_x / width, axis_size_y / height])


    max_stds = max(stds)

    dif = (max_stds) / 10.0
    if dif == 0:
        inte = 1
    else:
        inte = math.pow(10, math.floor(math.log10(dif)))
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte

    if r < 2.3:
        inte = inte * 2
    elif r < 7:
        inte = inte * 5
    else:
        inte = inte * 10
    vmax = inte * ((int)(max_stds / inte) + 1)

    std_list = np.arange(inte,vmax+inte/2,inte)

    corr_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9,0.98]
    #画观测弧线
    # 画弧线
    angle = np.arange(0,math.pi/2,math.pi/1000)
    std = stds[0]
    x2 = np.cos(angle) * std
    y2 = np.sin(angle) * std
    plt.plot(x2, y2,color = "steelblue",linewidth = 1)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std
        y2 = np.sin(angle) * std
        if i <len(std_list)-1:
            plt.plot(x2, y2, ":", color="k", linewidth=0.5)
        else:
            plt.plot(x2, y2,color = "k",linewidth = 0.5)


    #画围绕观测的弧线
    angle = np.arange(0,math.pi,math.pi/1000)
    for i in range(len(std_list)):
        std = std_list[i]
        x2 = np.cos(angle) * std + stds[0]
        y2 = np.sin(angle) * std
        dis = np.sqrt(x2 * x2 + y2 * y2)
        x2 = x2[dis < vmax]
        y2 = y2[dis < vmax]

        plt.plot(x2, y2, ":", color="g", linewidth=0.5)

    #相关系数射线
    r0 = np.arange(0,vmax,inte/100)
    for i in range(len(corr_list)):
        corr = corr_list[i]
        angle = np.arccos(corr)
        x1 = r0 * np.cos(angle)
        y1 = r0 * np.sin(angle)
        ax1.plot(x1, y1, '-.', color='b', linewidth=0.4)
        rt = vmax* 1.01
        xt = rt * np.cos(angle)
        yt = rt * np.sin(angle)
        ax1.text(xt, yt, str(corr),fontsize = sup_fontsize * 0.8)


    angle = 60 * math.pi/180
    rt = vmax * 1.01
    xt = rt * np.cos(angle)
    yt = rt * np.sin(angle)

    ax1.text(xt, yt,"相关系数" , fontsize=sup_fontsize * 0.8,rotation=-30,)

    ax1.set_xticks(std_list)
    ax1.set_yticks(std_list)
    ax1.set_xlim(0,vmax)
    ax1.set_ylim(0, vmax)
    ax1.spines['right'].set_visible(False)
    ax1.spines['top'].set_visible(False)
    ax1.set_xlabel("标准差")
    ax1.set_ylabel("标准差")



    # 画预报观测点
    labels = ["观测"]
    labels.extend(member_list)
    for i in range(len(stds)):
        corr = corrs1[i]
        angle = np.arccos(corr)
        xp = stds[i] * np.cos(angle)
        yp = stds[i] * np.sin(angle)
        ax1.plot(xp, yp,"o", label=labels[i], markersize=6)
    lines, label1 = ax1.get_legend_handles_labels()

    if len(stds)> 7:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.4, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    elif len(stds)>5:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.2, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    else:
        legend2 = ax1.legend(lines, label1, loc="upper right",
                         bbox_to_anchor=(1.1, 1.05), ncol=1, fontsize=sup_fontsize * 0.9)
    ax1.add_artist(legend2)


    title = title + "\n"
    ax1.set_title(title,fontsize = sup_fontsize)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()



def frequency_histogram_error(ob, fo,grade_list=None, member_list=None,  vmax = None,save_path=None,show = False,dpi = 300,plot = "bar", title="误差频率统计图",
                        sup_fontsize = 10,width = None,height = None,log_y = False):
    '''
    frequency_histogram 对比测试数据和实况数据的发生的频率
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :param save_path: 保存地址
    :return: 无
    '''
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape


    legend = []
    if member_list is None:
        if new_Fo_shape[0] <= 1:
            legend.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                legend.append('预报' + str(i + 1))
    else:
        legend.extend(member_list)

    error = new_Fo - ob

    result_array = meteva.method.frequency_table(ob, error, grade_list=grade_list)
    result_array = result_array[1:,:]

    total_count = np.sum(result_array[0,:])
    result_array /= total_count
    if grade_list is not None:
        if len(grade_list) >10:
            axis = ["<\n" + str(round(grade_list[0],6))]
            for index in range(len(grade_list)):
                axis.append(str(round(grade_list[index],6)))
            axis.append(">=\n" + str(round(grade_list[-1],6)))
        else:
            axis = ["<" + str(round(grade_list[0],6))]
            for index in range(len(grade_list) - 1):
                axis.append("[" + str(round(grade_list[index],6)) + "," + str(round(grade_list[index + 1],6)) + ")")
            axis.append(">=" + str(round(grade_list[-1],6)))


    else:
        new_fo = copy.deepcopy(fo).flatten()
        new_ob = copy.deepcopy(ob).flatten()
        fo_list = list(set(new_fo.tolist()))
        fo_list.extend(list(set(new_ob.tolist())))
        axis = list(set(fo_list))

    name_list_dict = {}
    name_list_dict["legend"] = legend
    name_list_dict["类别"] = axis
    if log_y:
        vmin = None
    else:
        vmin = 0
    if plot == "bar":
        meteva.base.plot_tools.bar(result_array,name_list_dict,ylabel= "样本占比",vmin = vmin,vmax = vmax,save_path = save_path,show = show,dpi = dpi,title=title,
                                   width = width,height = height,sup_fontsize= sup_fontsize,log_y = log_y)
    else:
        meteva.base.plot_tools.plot(result_array, name_list_dict, ylabel="样本占比", vmin=vmin, vmax=vmax, save_path=save_path,
                                   show=show, dpi=dpi, title=title,
                                    width = width,height = height,sup_fontsize= sup_fontsize,log_y = log_y)





def accumulation_change_with_strenght(ob,fo,member_list = None,save_path=None,  show = False,dpi = 300,title="降水量随强度变化图",
             sup_fontsize = 14,width = None,height = None,log_y = False,y_log = None,max_x = None):
    if y_log is not None:
        print(
            "warning: the argument y_log will be abolished, please use log_y instead\n警告：为保持和其它函数的名称一致，参数log_y将被废除，以后请使用参数log_y代替")
        log_y = y_log

    accu_stren = meteva.method.continuous.table.accumulation_strenght_table(ob,fo)
    min_not_zero = np.min(accu_stren[accu_stren>0])
    maxv = np.max(accu_stren)
    shape = accu_stren.shape
    grade = np.arange(1,shape[1]+1,1)
    nfo = shape[0]-1
    if width is None:
        width = 10
    if height is None:
        height = width *0.6

    fig = plt.figure(figsize=(width, height),dpi = dpi)

    if member_list is None:
        labels = ["观测"]
        for i in range(1,nfo+1):
            labels.append("预报"+str(i))
    else:
        labels = ["观测"]
        labels.extend(member_list)
    for line in range(len(labels)):
        plt.plot(grade,accu_stren[line], label=labels[line],marker = ".")
        plt.xlabel("降水强度(毫米/小时)", fontsize=0.9 * sup_fontsize)
        plt.ylabel("累计降水量", fontsize=0.9 * sup_fontsize)
        plt.title(title, fontsize=0.9 * sup_fontsize)
        if(log_y):
            ax_one = plt.gca()
            for tick in ax_one.yaxis.get_major_ticks():
                tick.label1.set_fontproperties('stixgeneral')
            plt.yscale('log')
        plt.xticks(fontsize=0.8 * sup_fontsize)
        plt.legend(loc="upper right")



    # 设置次刻度间隔
    if max_x is not None:
        maxx = max_x
    else:
        maxx = len(grade)
    if(maxx <20):
        xmi = 1
        Xmi = 1
    elif(maxx <50):
        xmi = 1
        Xmi = 5
    elif (maxx <100):
        xmi = 1
        Xmi = 10
    elif (maxx <300):
        xmi = 5
        Xmi = 20
    elif (maxx <1000):
        xmi = 10
        Xmi = 50
    else:
        xmi = 50
        Xmi = 200
    ax1 = plt.gca()
    xmajorLocator = mpl.ticker.MultipleLocator(Xmi)  # 将x主刻度标签设置为次刻度10倍
    ax1.xaxis.set_major_locator(xmajorLocator)
    xminorLocator = mpl.ticker.MultipleLocator(xmi)  # 将x轴次刻度标签设置xmi
    ax1.xaxis.set_minor_locator(xminorLocator)
    plt.xlim(0,maxx)
    if log_y:
        plt.ylim(min_not_zero,maxv * 3)
    else:
        plt.ylim(0,maxv * 1.1)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()
    return accu_stren

def frequency_change_with_strenght(ob,fo,member_list = None,save_path=None,  show = False,dpi = 300,title="降水频次随强度变化图",
             sup_fontsize = 14,width = None,height = None,log_y = False,y_log = None,max_x = None):

    if y_log is not None:
        print(
            "warning: the argument y_log will be abolished, please use log_y instead\n警告：为保持和其它函数的名称一致，参数log_y将被废除，以后请使用参数log_y代替")
        log_y = y_log

    accu_stren = meteva.method.continuous.table.frequency_strenght_table(ob,fo)
    min_not_zero = np.min(accu_stren[accu_stren>0])
    maxv = np.max(accu_stren)
    shape = accu_stren.shape
    grade = np.arange(1,shape[1]+1,1)
    nfo = shape[0]-1
    if width is None:
        width = 10
    if height is None:
        height = width *0.6

    fig = plt.figure(figsize=(width, height),dpi = dpi)

    if member_list is None:
        labels = ["观测"]
        for i in range(1,nfo+1):
            labels.append("预报"+str(i))
    else:
        labels = ["观测"]
        labels.extend(member_list)
    for line in range(len(labels)):
        plt.plot(grade,accu_stren[line], label=labels[line],marker = ".")
        plt.xlabel("降水强度(毫米/小时)", fontsize=0.9 * sup_fontsize)
        plt.ylabel("降水频次", fontsize=0.9 * sup_fontsize)
        plt.title(title, fontsize=0.9 * sup_fontsize)
        if(log_y):
            ax_one = plt.gca()
            for tick in ax_one.yaxis.get_major_ticks():
                tick.label1.set_fontproperties('stixgeneral')
            plt.yscale('log')
        plt.xticks(fontsize=0.8 * sup_fontsize)
        plt.legend(loc="upper right")



    # 设置次刻度间隔
    if max_x is not None:
        maxx = max_x
    else:
        maxx = len(grade)
    if(maxx <20):
        xmi = 1
        Xmi = 1
    elif(maxx <50):
        xmi = 1
        Xmi = 5
    elif (maxx <100):
        xmi = 1
        Xmi = 10
    elif (maxx <300):
        xmi = 5
        Xmi = 20
    elif (maxx <1000):
        xmi = 10
        Xmi = 50
    else:
        xmi = 50
        Xmi = 200
    ax1 = plt.gca()
    xmajorLocator = mpl.ticker.MultipleLocator(Xmi)  # 将x主刻度标签设置为次刻度10倍
    ax1.xaxis.set_major_locator(xmajorLocator)
    xminorLocator = mpl.ticker.MultipleLocator(xmi)  # 将x轴次刻度标签设置xmi
    ax1.xaxis.set_minor_locator(xminorLocator)
    plt.xlim(0,maxx)
    if log_y:
        plt.ylim(min_not_zero,maxv * 3)
    else:
        plt.ylim(0,maxv * 1.1)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()
    return accu_stren





#!/usr/bin/python3.6
# -*- coding:UTF-8 -*-
#添加一些常量
import pkg_resources
PI = 3.1415926
E = 2.7182818
ER = 6371.229
OMEGA = 0.00072722
IV = 999999
K = 273.15
dis_1_degree = 111.195
station_国家站 = pkg_resources.resource_filename('meteva', "resources/stations/sta2411_alt.txt")
station_国家站_考核区域站 = pkg_resources.resource_filename('meteva', "resources/stations/stat10461.txt")
station_全球城市 = pkg_resources.resource_filename('meteva', "resources/stations/station_global_alt_11621.txt")
station_全球重点城市 = pkg_resources.resource_filename('meteva', "resources/stations/sta_global_alt_243.txt")
terrain_height_grd = pkg_resources.resource_filename('meteva', "resources/stations/dem_0.00833.nc")
terrain_height_sta = pkg_resources.resource_filename('meteva', "resources/stations/sta_only_alt.txt")
gds_ip_port = None
customized_basemap_list = None
beaufort_scale = [0.0,0.3,1.6,3.4,5.5,8.0,10.8,13.9,17.2,20.8,24.5,28.5,32.7,37,41.5,46.2,51,56.1]
wave_grade = [0,0.01,0.1,0.5,1.3,2.5,4.0,6.0,9.0,14.0,100]
cmadaas_set = None
cimiss_set = None
plot_color_dict = None
ws_jar_path = None


from . import score
from . import plot

from .plot import  scatter_uv,scatter_uv_error,statisitic_uv,probability_density_uv
from .score import acd_nas,acs_nasws,acz_na,acs,acz,nas_d,nasws_s,na_ds,scd_nas,scd,scs,scs_nasws,acd
from .score import wind_weaker_rate,wind_severer_rate,wind_severer_rate_nasws,wind_weaker_rate_nasws
from .score import nasws_uv,na_uv,acd_uv,acs_uv,acz_uv,nas_uv,scd_uv,scs_uv,wind_severer_rate_uv,wind_weaker_rate_uv
from .score import distance_tdis,distance,tdis
from .score import tase_angle,tase_angle_uv,me_angle,me_angle_uv,mae_angle,mae_angle_uv,rmse_angle,rmse_angle_uv


#!/usr/bin/python
# -*- coding: utf-8 -*-
#Filename: SignGenUtil.py

'''
Created on Sep 14, 2020

@author: anduin
'''

import hashlib

class SignGenUtil():
    """description of class 
    get sign for url
    """
    
    @staticmethod
    def getSign(params):
        '''
                    生成sign标签
        '''
        try:
            paramString = ""
            if "params" in params:
                paramsVal = params.pop("params")
                keyValList = paramsVal.split("&")
                for keyVal in keyValList:
                    rsts = keyVal.split("=")
                    if len(rsts) > 1:
                        params[rsts[0]] = rsts[1]
                    else:
                        params[rsts[0]] = ""
                        
            keys = sorted(params)
            for key in keys:
                paramString = paramString + key + "=" + str(params.get(key)) + "&"

            if(paramString):
                paramString = paramString[:-1]
            
            #进行MD5运算
            return hashlib.md5(paramString.encode(encoding='UTF-8')).hexdigest().upper()
        except Exception as data:
            print(str(data))
            raise data
            #return None






# -*-coding:utf-8-*-
import numpy as np
from .feature_axis import feature_axis


def feature_props(look,label,ob_or_fo = None,which_comps=None, q=None):
    if ob_or_fo =="ob":
        tmp = look['grd_ob_features'][label]
    elif ob_or_fo == "fo":
        tmp = look['grd_fo_features'][label]
    else:
        if label in look["grd_features"].keys():
            tmp = look['grd_features'][label]
        else:
            return None
    grid0 = look["grid"]
    #x = np.zeros((grid0.nlat,grid0.nlon))
    #print("****")
    #print(tmp)
    #x[tmp] = 1
    #x = copy.deepcopy(tmp)
    if which_comps is None:
        which_comps = ["centroid", "area", "axis", "intensity"]
    if q is None:
        q = [0,0.05,0.1,0.25,0.5, 0.75,0.9,0.95,1]
    out = {}

    if "centroid" in which_comps:
        #xd = x.shape
        #dim0 = xd[0]
        #dim1 = xd[1]
        #dim0 = grid0.nlat
        #dim1 = grid0.nlon
        #range0 = np.tile(np.arange(dim0), dim1)
        #range1 = (np.arange(dim1)).repeat(dim0)
        #loc = np.stack((range0, range1), axis=-1)
        #xbool = np.reshape(x, x.size, 'F')
        #xcen = grid0.slon + np.mean(loc[:, 0][xbool == 1]) * grid0.dlon
        #ycen = grid0.slat + np.mean(loc[:, 1][xbool == 1]) * grid0.dlat
        xcen = grid0.slon + np.mean(tmp[1]) * grid0.dlon
        ycen = grid0.slat + np.mean(tmp[0]) * grid0.dlat
        out['centroid'] = {"x": xcen, "y": ycen}
    if "area" in which_comps:
        #out["area"] = np.sum(x) * grid0.dlon*grid0.dlat
        out["area"] = len(tmp[0]) * grid0.dlon * grid0.dlat
    if "axis" in which_comps:
        out["axis"] = feature_axis(look,label,ob_or_fo = ob_or_fo, fac = grid0.dlon*grid0.dlat)
    if "intensity" in which_comps:
       #ivec = {}
       # df = pd.DataFrame(np.array(im[x]), columns=q)
       # for i, val in q:
       #     ivec[val] = df.quantile(val)
        if ob_or_fo == "ob":
            values = look['grd_ob'].values.squeeze()
            labels = look["grd_ob_label"].values.squeeze()
        elif ob_or_fo == "fo":
            values = look['grd_fo'].values.squeeze()
            labels = look["grd_fo_label"].values.squeeze()
        else:
            values = look['grd'].values.squeeze()
            labels = look["grd_label"].values.squeeze()
        valid = values[labels == label]
        ivec = np.quantile(valid,q)
        out["intensity"] = ivec
    return out



#!/usr/bin/python
# -*- coding: utf-8 -*-
#Filename: httpconn.py

'''
Created on Sep 14, 2020

@author: anduin
'''

import zlib
from urllib.parse import urlparse
from urllib.parse import urljoin
from urllib.parse import urlencode
from http.client import HTTPConnection
from http.client import HTTPSConnection
import json
import urllib.request
import ssl
ssl._create_default_https_context = ssl._create_unverified_context

from .loglib import *

class HttpConn():
    @staticmethod
    def joinUrl(url, path):
        return urljoin(url, path)
    
    @staticmethod
    def parseUrl(url):
        try:
            prse = urlparse(url)
            ishttp = False
            if prse.scheme.upper() == 'HTTP':
                ishttp = True

            start = url.find(prse.netloc)
            path = url[start + len(prse.netloc):]
            if len(path) == 0:
                path = r'/'
                
            return prse.netloc, path, ishttp
        except Exception as data:
            LogLib.logger.error('parseUrl except ' + str(data))
            print(data)
            return '', '', ''
    
    
    @staticmethod
    def getHeaderValue(headers, name):
        for header in headers:
            if header[0].lower() == name.lower():
                return header[1]
            
        return None
    
    @staticmethod
    def getHeaderValues(headers, name):
        values = []
        for header in headers:
            if header[0].lower() == name.lower():
                values.append(header[1])
            
        return values
    
    @staticmethod
    def getWebPage(host, url, headers = None, ishttp = True):
        conn = None
        try:
            if ishttp:
                conn=HTTPConnection(host, timeout = 60)
            else:
                conn=HTTPSConnection(host, timeout = 60)
            
            if headers == None:
                conn.request('GET', url)
            else:
                conn.request('GET', url, headers = headers)
            result = conn.getresponse()
            rststatus = result.status
            rstheaders = result.getheaders()
            rstcontent_org = result.read()
            
            encoding = HttpConn.getHeaderValue(rstheaders, 'content-encoding')
            if encoding != None and encoding.lower() == 'gzip':
                decomp = zlib.decompressobj(16+zlib.MAX_WBITS)
                rstcontent = decomp.decompress(rstcontent_org)
            else:
                rstcontent = rstcontent_org
##            if __name__ == '__main__':
##                print host
##                print url
##                print rststatus
##                print rstheaders
##                print rstcontent
                
            conn.close()
            
            return (rststatus, rstcontent, rstheaders)
        except Exception as data:
            LogLib.logger.error('getWebPage except ' + str(data))
            if conn != None:
                conn.close()
            
            return (0, None, None)
        
    @staticmethod
    def postData(host, url, headers, params, ishttp = True):
        conn = None
        try:
            if ishttp:
                conn=HTTPConnection(host, timeout = 60)
            else:
                conn=HTTPSConnection(host, timeout = 60)
                
            if headers == None:
                conn.request('POST', url, body = params)
            else:
                conn.request('POST', url, body = params, headers = headers)
            
            result = conn.getresponse()
            rststatus = result.status
            rstheaders = result.getheaders()
            rstcontent_org = result.read()
            
            encoding = HttpConn.getHeaderValue(rstheaders, 'content-encoding')
            if encoding != None and encoding.lower() == 'gzip':
                decomp = zlib.decompressobj(16+zlib.MAX_WBITS)
                rstcontent = decomp.decompress(rstcontent_org)
            else:
                rstcontent = rstcontent_org
                
            conn.close()
            
            return (rststatus, rstcontent, rstheaders)
        except Exception as data:
            LogLib.logger.error('postData except ' + str(data))
            if conn != None:
                print('conn is None')
                conn.close()
            
            return (0, None, None)
    
    @staticmethod
    def getWebPageWithProxy(url, proxyhost, proxyport, headers = {}):
        try:
            proxyurl = "http://" + proxyhost + ":" + "%d"%proxyport
            proxy_support = urllib.request.ProxyHandler({"http": proxyurl, "https": proxyurl})
            opener = urllib.request.build_opener(proxy_support)
            urllib.request.install_opener(opener)
            req = urllib.request.Request(url)
            if headers != None:
                for header in headers:
                    req.add_header(header, headers[header])
                
            result = urllib.request.urlopen(req)
            rststatus = result.status
            rstheaders = result.getheaders()
            rstcontent_org = result.read()
            
            encoding = HttpConn.getHeaderValue(rstheaders, 'content-encoding')
            if encoding != None and encoding.lower() == 'gzip':
                decomp = zlib.decompressobj(16+zlib.MAX_WBITS)
                rstcontent = decomp.decompress(rstcontent_org)
            else:
                rstcontent = rstcontent_org
##            if __name__ == '__main__':
##                print host
##                print url
##                print rststatus
##                print rstheaders
##                print rstcontent
                
            return (rststatus, rstcontent, rstheaders)
        except Exception as data:
            LogLib.logger.error('getWebPageWithProxy except ' + str(data))
            
            return (0, None, None)
        
    @staticmethod
    def postDataWithProxy(url, proxyhost, proxyport, params, headers = {} ):
        try:
            proxyurl = "http://" + proxyhost + ":" + "%d"%proxyport
            proxy_support = urllib.request.ProxyHandler({"http": proxyurl, "https": proxyurl})
            opener = urllib.request.build_opener(proxy_support)
            urllib.request.install_opener(opener)
            pdata = params.encode(encoding='UTF8')
            req = urllib.request.Request(url, pdata, headers, method='POST')
                
            result = urllib.request.urlopen(req)
            rststatus = result.status
            rstheaders = result.getheaders()
            rstcontent_org = result.read()
            
            encoding = HttpConn.getHeaderValue(rstheaders, 'content-encoding')
            if encoding != None and encoding.lower() == 'gzip':
                #compressedstream = StringIO.StringIO(rstcontent_org)
                #gzipper = gzip.GzipFile(fileobj=compressedstream)      
                #rstcontent = gzipper.read()
                decomp = zlib.decompressobj(16+zlib.MAX_WBITS)
                rstcontent = decomp.decompress(rstcontent_org)
            else:
                rstcontent = rstcontent_org
                
            return (rststatus, rstcontent, rstheaders)
        except Exception as data:
            LogLib.logger.error('postDataWithProxy except ' + str(data))
            
            return (0, None, None)
    
    @staticmethod
    def downloadFileWithWget(url, fullpath):
        try:
            import subprocess
            res = subprocess.Popen('wget ' + '"' + url + '" -O ' + fullpath, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
            return res.stdout.readlines()
        except Exception as data:
            return None

if __name__ == '__main__':
    print(HttpConn.downloadFileWithWget(r'https://confluence.ecmwf.int/download/attachments/8650755/ecFlow-5.5.3-Source.tar.gz?api=v2', r'c:\workdoc\ecFlow-5.5.3-Source.tar.gz'))


    print('test done')


# _*_ coding: utf-8 _*_

from os import path
from setuptools import find_packages, setup
from codecs import open
name = "meteva"
author ="liucouhua,daikan,wangbaoli,tangbuxing"
version =__import__(name).__version__

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name=name,

    version=version,

    description=("A collections of functions for meteorological verification."),
    long_description=long_description,

    # author
    author=author,
    author_email='liucouhua@163.com',

    # LICENSE
    license='GPL3',

    classifiers=[
      'Development Status :: 3 - Alpha',
      'Intended Audience :: Developers',
      'Programming Language :: Python :: 3',
    ],

    packages=find_packages(exclude=['docs', 'tests', 'build', 'dist']),
    include_package_data=True,
    exclude_package_data={'': ['.gitignore']},

    install_requires=[
                      'numpy>=1.12.1',
                      'pandas>=1.0.4',
                      "netCDF4>=1.4.2",
                      'scipy>=0.19.0',
                      'xarray>=0.10.0',
                      'scikit-learn>=0.21.2',
                      'matplotlib>=3.0.0',
                      "httplib2>=0.12.0",
                      "protobuf<3.20.0",
                      "pyshp>=2.1.0",
                      "tables>=3.4.4",
                      "urllib3>=1.21.1"
                      #"JPype1>=1.4.0"
                      ]
)

# install_requires = ['numpy>=1.12.1',
#                     'scipy>=0.19.0',
#                     'pandas>=1.0.1',
#                     'xarray>=0.11.0',
#                     'scikit-learn>=0.21.2',
#                     'matplotlib>=3.3.1',
#                     "httplib2>=0.12.0",
#                     "protobuf>=3.6.1",
#                     "netCDF4>=1.4.2",
#                     "seaborn>=0.9.0",
#                     "pyshp>=2.1.0",
#                     "tables>=3.4.4",
#                     "urllib3>=1.21.1",
#                     "statsmodels>=0.11.0",
#                     ]


#!/usr/bin/env python
# coding=utf8
import urllib3
import httplib2
import meteva

def get_http_result (host, port, url):
    http_client = None
    try:
        #http_client = httplib.HTTPConnection(host, port, timeout=120)
        #http_client.request('GET', url)
        #response = http_client.getresponse()
        
        http_client = httplib2.Http()
        url = "http://"+str(host)+":"+str(port)+str(url)
        response,content = http_client.request(url,'GET')

        return response.status,  content
    except Exception as e:
        print(e)
        return 0,
    #finally:
    #    if http_client:
    #        http_client.close()


def get_http_result_cimiss(interface_id, params, data_format='json',show_url = False):
    """
    Get the http result from CIMISS REST api service.
    :param interface_id: MUSIC interface id.
    :param params: dictionary for MUSIC parameters.
    :param data_format: MUSIC server data format.
    :return:
    """


    dns = meteva.base.cimiss_set[0]
    user_id = meteva.base.cimiss_set[1]
    pwd = meteva.base.cimiss_set[2]

    # construct url
    url = 'http://' + dns + '/cimiss-web/api?userId=' + user_id + \
          '&pwd=' + pwd + '&interfaceId=' + interface_id

    # params
    for key in params:
        url += '&' + key + '=' + params[key].strip()

    # data format
    url += '&dataFormat=' + data_format
    if show_url:print(url)
    # request http contents
    http = urllib3.PoolManager()
    req = http.request('GET', url)
    if req.status != 200:
        req = http.request('GET', url)
        if req.status != 200:
            print('Can not access the url: ' + url)
            return None
    return req.data


import meteva
import pandas as pd
import numpy as np
'''
#该模块将逐步包含业务规范中的短时强降水的检验规则
'''

def edition_2020_1(sta_ob,sta_fo_list,pcapital_fo_list, id_province_dict):
    '''

    :param sta_ob: 观测降水站点数据合集,其中站点值为具体的降水量值
    :param sta_fo_list:列表，其中第0个元素为中央台的预报插值到站点得到的站点数据的合集，第1个元素为省台预报对应结果.
     其中的站点值为0或1,0代表预报短时强降水不发生，1代表预报短时强降水发生
    :param pcapital_fo_list:列表，其中第0个元素为中央台的预报插值到省会城市得到的站点数据的合集，第1个元素为省台省会预报.
     其中的站点值为0或1,0代表预报短时强降水不发生，1代表预报短时强降水发生
    :param id_provice_dict: 字典形式数据，用于标记每个站号对应的省份
    :return:
    '''
    # 业务标准相关的参数
    west_province_list = ["内蒙古","西藏","青海","宁夏","新疆"]
    west_threshold = 10  #西部省份短时强降水阈值
    other_threshold = 20 #其它省份短时强降水阈值
    R = 40               #邻域半径

    # 设定每个站点对应的阈值
    id_list = []
    threshold_list = []

    for id in id_province_dict:
        id_list.append(id)
        if id_province_dict[id] in west_province_list:
            threshold_list.append(west_threshold)
        else:
            threshold_list.append(other_threshold)
    id_threshold_dict = {"id":id_list,"thre":threshold_list}
    id_threshold_sta = meteva.base.sta_data(pd.DataFrame(id_threshold_dict))


    #根据邻域算法根据包含具体降水量的观测数据，计算出每个站点邻域内是否发生了短时强降水，0代表没发生，1代表发生了；
    sta_ob_01 = meteva.method.p2a_vto01(sta_ob,r = R,threshold= id_threshold_sta)

    #在将观测和预报数据进行匹配对齐以及合并,其中观测值缺省的样本会被删除
    sta_all_01 = meteva.base.combine_on_obTime_id(sta_ob_01,sta_fo_list,need_match_ob=True)

    #在将观测和省会预报数据进行匹配对齐以及合并,其中观测值缺省的样本会被删除
    sta_all_01_pcapital = meteva.base.combine_on_obTime_id(sta_ob_01,pcapital_fo_list,need_match_ob=True)


    #按时效分类检验，所有站点
    pod_results = meteva.product.score(sta_all_01,meteva.method.pod,g = "dtime")[0]
    far_results = meteva.product.score(sta_all_01,meteva.method.far,g = "dtime")[0]

    #按时效分类检验，针对省会城市预报
    pod_results_pcapital = meteva.product.score(sta_all_01_pcapital,meteva.method.pod,g = "dtime")[0]
    far_results_pcapital = meteva.product.score(sta_all_01_pcapital,meteva.method.far,g = "dtime")[0]


    #计算预报技巧
    spo = np.zeros(12)  # 格点1小时命中率预报技巧(所有站点 )
    sfa = np.zeros(12)  # 格点1小时空报率预报技巧(所有站点 )
    spo_pcapital = np.zeros(12)   # 单站1小时命中率预报技巧(省会城市)
    sfa_pcapital = np.zeros(12)    # 单站1小时空报率预报技巧(省会城市)
    #循环12个时效
    for i in range(12):
        spo[i] = meteva.method.spo(pod_results[i, 1], pod_results[i, 0])    # meteva.method.yes_or_no.skill.spo()
        sfa[i] = meteva.method.sfa(far_results[i, 1], far_results[i, 0])    # meteva.method.yes_or_no.skill.sfa()
        spo_pcapital[i] = meteva.method.spo(pod_results_pcapital[i, 1], pod_results_pcapital[i, 0])
        sfa_pcapital[i] = meteva.method.sfa(far_results_pcapital[i, 1], far_results_pcapital[i, 0])

    return spo,sfa,spo_pcapital,sfa_pcapital



if __name__ == "__main__":

    station_file = r"H:\test_data\station_pta.txt"  #  读取自动站的站号和经纬度信息
    station = meteva.base.read_station(station_file)
    station.iloc[:,-1] = meteva.base.IV

    dict0 = meteva.base.station_id_name_dict
    dict1 = {}
    for id in dict0.keys():
        str0 = dict0[id]
        strss = str0.split("_")
        str1 = strss[0]
        dict1[id] = str1

    import datetime

    time_start = datetime.datetime(2020,8,18,8,0)
    time_end = datetime.datetime(2020,8,19,8,0)

    ###读取收集观测数据
    dir_ob = r"Y:\qpe\autorun\data_all\001h\YYYYMMDDHH.000.001h"
    sta_list = []
    time0 = time_start
    while time0 <= time_end:
        path = meteva.base.get_path(dir_ob,time0)
        #meteva.base.print_gds_file_values_names(path)
        sta = meteva.base.read_stadata_from_micaps3(path,station = station,time = time0,dtime = 0,level = 0,data_name = "ob",show = True)
        sta_list.append(sta)
        time0 += datetime.timedelta(hours = 1)
    ob_sta_all = pd.concat(sta_list,axis = 0)  #数据拼接

    # scmoc
    dir_scmoc = r"O:\data\grid\NWFD_SCMOC_1H\RAIN01\YYYYMMDD\YYMMDDHH.TTT.nc"
    sta_list = []
    time0 = time_start
    while time0 < time_end:
        for dh in range(1, 13):
            path = meteva.base.get_path(dir_scmoc, time0, dh)
            grd = meteva.base.read_griddata_from_nc(path,time=time0,dtime = dh,level = 0,data_name="scmoc",show=True)
            if grd is not None:
                sta = meteva.base.interp_gs_linear(grd, station)
                sta_list.append(sta)
        time0 += datetime.timedelta(hours=12)  # 此处跳着读是为了减少测试用时
    scmoc_sta_all = pd.concat(sta_list, axis=0)  # 数据拼接
    scmoc_sta_all.iloc[:,-1] = 0 + (scmoc_sta_all.iloc[:,-1] >= 2)  # 将预报转换成0-1形式

    # scmoc
    dir_gmosrr = r"O:\data\grid\GMOSRR\ROLLING_UPDATE\RAIN01\YYYYMMDD\YYMMDDHH.TTT.nc"
    sta_list = []
    time0 = time_start
    while time0 < time_end:
        for dh in range(1, 13):
            path = meteva.base.get_path(dir_gmosrr, time0, dh)
            grd = meteva.base.read_griddata_from_nc(path,time=time0,dtime = dh,level = 0,data_name="gmosrr",show=True)
            if grd is not None:
                sta = meteva.base.interp_gs_linear(grd, station)
                sta_list.append(sta)
        time0 += datetime.timedelta(hours=12)  # 此处跳着读是为了减少测试用时
    gmosrr_sta_all = pd.concat(sta_list, axis=0)  # 数据拼接
    gmosrr_sta_all.iloc[:, -1] = 0 + (gmosrr_sta_all.iloc[:, -1] >= 3)  # 将预报转换成0-1形式

    #读取城镇预报，原则上从其它文件中读取，但为了简化问题，此处从网格预报中提取

    pcapital_id = [54511]

    scmoc_sta_pcapital = meteva.base.in_id_list(scmoc_sta_all,pcapital_id)
    gmosrr_sta_pcapital = meteva.base.in_id_list(gmosrr_sta_all,pcapital_id)

    #

    spo, sfa, spo_pcapital, sfa_pcapital = edition_2020_1(ob_sta_all,[scmoc_sta_all,gmosrr_sta_all],
                                                     [scmoc_sta_pcapital,gmosrr_sta_pcapital],id_province_dict=dict1)


    print(spo)
    print(sfa)
    print(spo_pcapital)
    print(sfa_pcapital)


# -*- coding: utf-8 -*-
"""
Created on Thu Aug  6 10:53:50 2020

@author: 1
"""
import numpy as np
from scipy import ndimage
import sys


def censqdelta(x, y, N, const, p):

    #检查x、y是否二值化
    
    xdim = np.shape(x)
    rep00_1 = np.arange(1, xdim[0] + 1, 1)
    rep00 = np.tile(rep00_1, xdim[1])
    rep0_1 = np.arange(1, xdim[1] + 1, 1)
    rep0 = rep0_1.repeat(xdim[0], axis = 0)    #按行进行元素重复
    loc = np.vstack((rep00, rep0)).T    #数组转置array.T

    if (N == None):
        N = max(xdim)
    if (N % 2 == 0):
        N = N + 1


    xy = (x>0)|(y>0)
    
    #计算x,y合并后的形心:目标区域的行列求和除以总的行列求和

    xyloc_x = np.argwhere(xy == 1)[:, 0]    #合并以后图形的x坐标
    xyloc_y = np.argwhere(xy == 1)[:, 1]    #合并以后图形的y坐标
    xycen = np.array([sum(xyloc_x)/len(xyloc_x), sum(xyloc_y)/len(xyloc_y)])    #两个object合并以后的形心坐标
    
    
    #robjects.r('''
    #       f <- function(N){cbind(rep(1:N, N), rep(1:N, each = N))}
    #       ''')
    #bigDloc = np.array(robjects.r['f'](int(N)))    # N必须是int
    bigDcen = np.array((N-1)/2 + 1).repeat(2,axis = 0).reshape(1, 2)
    cendiff = np.array(bigDcen - xycen).reshape(1, 2)
    xloc = np.argwhere(x == 1)
    yloc = np.argwhere(y == 1)
    idX = xloc + cendiff.repeat(len(xloc),axis = 0)    #将cendiff变为矩阵，len = len(xloc)
    idY = yloc + cendiff.repeat(len(yloc),axis = 0)
    #勿删
    #goodIDx = np.array([j for i in idX for j in i if j >= 1 and j <= N])
    #goodIDy = np.array([j for i in idY for j in i if j >= 1 and j <= N])
    goodIDx = np.where((idX >= 1)&(idX <= N),True,False)    #多条件查询，满足条件返回True,不满足条件返回False
    goodIDy = np.where((idY >= 1)&(idY <= N),True,False)    #多条件查询，满足条件返回True,不满足条件返回False

    if (goodIDx.any() == False):
        print("warning message:\ncensqdelta: centering pushes observed data outside of new domain.  Removing some data.  Maybe choose larger N?")
        if (goodIDx.all() == False):
            try:
                sys.exit(0)
            except:
                print("censqdelta: No observed data remains after centering in this domain.")
        idX = np.array([j for i in idX for j in i if j == True])

    if (goodIDy.any() == False):
        print("warning message:\ncensqdelta: centering pushes forecast data outside of new domain.  Maybe choose larger N?")
        if (goodIDy.all() == False):
            try:
                sys.exit(0)
            except:
                print("censqdelta: No forecast data remains after centering in this domain.")
        idY = np.array([j for i in idY for j in i if j == True])    

    X = np.zeros(shape = (N, N), dtype = float)    #大小为N的0矩阵
    Y = np.zeros(shape = (N, N), dtype = float)
    #print(idX.shape)
    idx1 = idX[:,0].astype(np.int16)
    idx1[idx1<0] = 0
    idx1[idx1 >= N-1] = N-1
    idx2 = idX[:,1].astype(np.int16)
    idx2[idx2<0] = 0
    idx2[idx2 >= N-1] = N-1
    X[idx2,idx2] = 1

    #for j in np.trunc(idX).astype(int).tolist():
        # 将计算后的索引位置重新赋值为1
    #    X[j[0],j[1]] = 1


    idy1 = idY[:,0].astype(np.int16)
    idy1[idy1<0] = 0
    idy1[idy1 >= N-1] = N-1
    idy2 = idY[:,1].astype(np.int16)
    idy2[idy2<0] = 0
    idy2[idy2 >= N-1] = N-1
    Y[idy2,idy2] = 1

    #for k in np.trunc(idY).astype(int).tolist():
        #print(k)
    #    Y[k[0],k[1]] = 1
        
    #deltametric函数
    dA = ndimage.morphology.distance_transform_edt(1-X)     #distmap函数
    dB = ndimage.morphology.distance_transform_edt(1-Y)     #distmap函数
    if float('inf') == p:
        Z = abs(dA - dB)
        delta = np.max(Z)
    else:
        Z = abs(dA - dB) ** p
        iZ = np.mean(Z)
        delta = iZ**(1/p)

    return delta
        









        
        
        
        


import datetime
import meteva
import shutil
import os
import numpy as np


para_list_example = [
    {
        "input":r"O:\data\grid\GRAPES_GFS\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "output":r"L:\luoqi\GRAPES_GFS\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "start":datetime.datetime(2020,3,1,8,0),
        "end":datetime.datetime(2020,6,1,8,0),
        "hour_range":[8,21,12],
        "dh_range":[3,241,3],
        "recover":False
    },

    {
        "input": r"O:\data\grid\ECMWF_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "output": r"L:\luoqi\ECMWF_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "start": datetime.datetime(2020, 3, 1, 8, 0),
        "end": datetime.datetime(2020, 6, 1, 8, 0),
        "hour_range": [8, 21, 12],
        "dh_range": [3, 241, 3],
        "recover": False
    },

    {
        "input": r"O:\data\grid\NCEP_GFS_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "output": r"L:\luoqi\NCEP_GFS_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "start": datetime.datetime(2020, 3, 1, 8, 0),
        "end": datetime.datetime(2020, 6, 1, 8, 0),
        "hour_range": [8, 21, 12],
        "dh_range": [3, 241, 3],
        "recover": False
    },

    {
        "input": r"O:\data\grid\SHANGHAI_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "output": r"L:\luoqi\SHANGHAI_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "start": datetime.datetime(2020, 3, 1, 8, 0),
        "end": datetime.datetime(2020, 6, 1, 8, 0),
        "hour_range": [0, 24, 1],
        "dh_range": [0, 25, 1],
        "recover": False
    },

    {
        "input": r"O:\data\grid\GRAPES_MESO_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "output": r"L:\luoqi\GRAPES_MESO_HR\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "start": datetime.datetime(2020, 3, 1, 8, 0),
        "end": datetime.datetime(2020, 6, 1, 8, 0),
        "hour_range": [2, 24, 3],
        "dh_range": [0, 85, 1],
        "recover": False
    },

    {
        "input": r"O:\data\grid\GRAPES_3KM\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "output": r"L:\luoqi\GRAPES_3KM\APCP\YYYYMMDD\YYMMDDHH.TTT.nc",
        "start": datetime.datetime(2020, 3, 1, 8, 0),
        "end": datetime.datetime(2020, 6, 1, 8, 0),
        "hour_range": [2, 24, 3],
        "dh_range": [0, 37, 1],
        "recover": False
    },

]




def copy_data_file(para_list):
    for para in para_list:
        time0 = para["start"]
        time1 = datetime.datetime(time0.year,time0.month,time0.day,0,0)
        hour_list =  np.arange(para["hour_range"][0],para["hour_range"][1],para["hour_range"][2]).tolist()
        dh_list = np.arange(para["dh_range"][0], para["dh_range"][1], para["dh_range"][2]).tolist()
        recover = para["recover"]
        while time1 <= para["end"]:
            for hour in hour_list:
                time2 = time1 +datetime.timedelta(hours=hour)
                for dh in dh_list:
                    path_out = meteva.base.get_path(para["output"],time2,dh)
                    if recover or not os.path.exists(path_out):
                        path_in = meteva.base.get_path(para["input"],time2,dh)
                        if os.path.exists(path_in):
                            meteva.base.creat_path(path_out)
                            shutil.copyfile(path_in,path_out)
            time1 = time1 + datetime.timedelta(hours=24)


def copy_during_data_file(root_in,root_out,start = None,end = None,recover = False):
    path_list = meteva.base.path_tools.get_during_path_list_in_dir(root_in,None,start,end)
    for path in path_list:
        path_copy = path.replace(root_in,root_out)
        if not os.path.exists(path_copy) or recover:
            meteva.base.path_tools.creat_path(path_copy)
            shutil.copy(path,path_copy)

if __name__ == '__main__':
    #copy_data_file(para_list1)
    pass



import meteva
import os
import urllib.request

'''
def terrain_height_correction(sta_all,start = 1):
    delta = meteva.base.read_stadata_from_micaps3(meteva.base.terrain_height_correction_data)
    ids = list(set(sta_all["id"].values.tolist()))
    delta = meteva.base.in_id_list(delta,ids)
    #print(delta)
    delta["level"] = meteva.base.IV
    delta["time"] = meteva.base.IV
    delta["dtime"] = meteva.base.IV
    datanames = meteva.base.get_stadata_names(sta_all)
    sta_all_delta = meteva.base.combine_expand_IV(sta_all,delta)
    for i in range(start,len(datanames)):
        name = datanames[i]
        sta_all_delta[name] += sta_all_delta.iloc[:,-1]
    datanames = meteva.base.get_stadata_names(sta_all_delta)
    sta_all_corrected = sta_all_delta.drop(columns = [datanames[-1]])
    return sta_all_corrected
'''

def terrain_height_correct(sta_temp, grid, sta_alt = None, member_list =  None,rate = 0.6):
    '''

    :param sta_temp:
    :param grid:
    :param sta_alt:
    :param member_list:
    :param rate:
    :return:
    '''
    grd_alt_path = meteva.base.terrain_height_grd
    if not os.path.exists(grd_alt_path):
        url = "https://github.com/nmcdev/meteva/raw/master/meteva/resources/stations/dem_0.00833.nc"
        try:
            print("开始从github下载地形高度数据，请稍等")
            urllib.request.urlretrieve(url, filename=grd_alt_path)
        except Exception as e:
            print("从github下载地形高度数据失败，请重试，或者手动从\n"+url+"\n下载文件并保存至\n"+grd_alt_path)
            print(e)


    grd_alt = meteva.base.read_griddata_from_nc(grd_alt_path)
    grd_alt.values[grd_alt.values<0] = 0
    grid_alt = meteva.base.get_grid_of_data(grd_alt)
    grid_inner = meteva.base.get_inner_grid(grid,grid_alt)
    grd_alt = meteva.base.interp_gg_linear(grd_alt,grid_inner)
    if sta_alt is None:
        sta_alt = meteva.base.read_sta_alt_from_micaps3(meteva.base.station_国家站)
    sta_alt = meteva.base.in_grid(sta_alt,grid_inner)
    alt_grd_to_sta = meteva.base.interp_gs_linear(grd_alt, sta_alt)

    delta = sta_alt.copy()
    delta.iloc[:, -1] = (alt_grd_to_sta.iloc[:, -1] - sta_alt.iloc[:, -1]) * rate/100

    ids = list(set(sta_temp["id"].values.tolist()))
    delta = meteva.base.in_id_list(delta, ids)
    delta["level"] = meteva.base.IV
    delta["time"] = meteva.base.IV
    delta["dtime"] = meteva.base.IV
    datanames = meteva.base.get_stadata_names(sta_temp)

    sta_all_delta = meteva.base.combine_expand_IV(sta_temp, delta)
    if member_list is None:member_list = datanames
    for i in range(len(member_list)):
        name = member_list[i]
        sta_all_delta[name] += sta_all_delta.iloc[:, -1]
    datanames = meteva.base.get_stadata_names(sta_all_delta)

    sta_all_corrected = sta_all_delta.drop(columns=[datanames[-1]])
    return sta_all_corrected


#!/usr/bin/python
# -*- coding: utf-8 -*-
#Filename: clienthandler.py

'''
Created on Aug 23, 2020

@author: anduin
'''

import logging
import threading
import time

class LogData():
    def __init__(self, cid, logstr):
        self.clientid = cid
        self.logstr = logstr
        
class ClientHandler(logging.Handler):
    def __init__(self, clientmgr):
        logging.Handler.__init__(self)
        
        self.clientmgr = clientmgr
        
        self.logbuf = []
        
        self.con = threading.Condition()
        self.stopit = False
        self.logThread = threading.Thread(target=ClientHandler.threadproc, args=(self,))
        self.logThread.start()
        
    def __del__(self):
        #logging.Handler.__del__(self)
        self.stopit = True
        time.sleep(2)
    
    def sendlog(self):
        while not self.stopit:
            if len(self.logbuf) == 0:
                time.sleep(1)
                continue
                
            tmpbuf = []
            self.con.acquire()
            try:
                while len(tmpbuf) < 5 and len(self.logbuf) > 0:
                    tmpbuf.append(self.logbuf.pop(0))
            except Exception as data:
                print('clienthandle sendlog read from logbuf exception:%s' % (str(data)))
            self.con.release()
            
            
            try:
                for log in tmpbuf:
                    self.clientmgr.addLog(log.clientid, log.logstr)
            except Exception as data:
                print('clienthandle sendlog add log to client exception:%s' % (str(data)))
            
    @staticmethod
    def threadproc(chandler):
        time.sleep(5)
        chandler.sendlog()
            
    def emit(self, record):
        """
        Emit a record.

        Send the record to the Web server as a percent-encoded dictionary
        """
        try:
            clientid = 0
            index = record.msg.find(' ')
            if index != -1:
                str = record.msg[:index]
                try:
                    clientid = int(str)
                    record.msg = record.msg[index + 1:]
                except:
                    clientid = 0
                    #record.msg = '0 ' + record.msg
            #else:
            #    record.msg = '0 ' + record.msg
                
            #self.clientmgr.addLog(clientid, self.format(record))
            self.con.acquire()
            try:
                if len(self.logbuf) < 200:
                    self.logbuf.append(LogData(clientid, self.format(record)))
            except Exception as data:
                print('clienthandle exception:%s' % (str(data)))
                
            self.con.release()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

        
if __name__ == '__main__':
    from dispatch.clientmgr import ClientMgr
    cm = ClientMgr()
    ch = ClientHandler(cm)
    
    print('test done')
    



import meteva
import numpy as np
import pandas as pd
import datetime


def mean_of_sta(sta,used_coords = ["member"],span = 24,equal_weight = False,keep_all = True):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    sta1 = meteva.base.not_IV(sta)

    if used_coords == ["member"]:
        sta_mean = sta1.loc[:,meteva.base.get_coord_names()]
        sta_data = sta1[meteva.base.get_stadata_names(sta1)]
        value = sta_data.values
        mean = np.mean(value,axis=1)
        sta_mean['mean'] = mean
        return sta_mean
    elif used_coords == ["time"]:
        times = sta.loc[:, 'time'].values
        times = list(set(times))
        times.sort()
        times = np.array(times)
        dtimes = times[1:] - times[0:-1]
        min_dtime = np.min(dtimes)
        min_dhour = min_dtime / np.timedelta64(1, 'h')

        if span is None:

            rain_ac = meteva.base.in_time_list(sta, [times[0]])
            rain_ac["count_for_add"] = 1
            meteva.base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1, len(times)):
                rain01 = meteva.base.in_time_list(sta, times[i])
                rain01["count_for_add"] = 1
                meteva.base.set_stadata_coords(rain01, time=times[-1])
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain01, how="outer", default=0)

            names = meteva.base.get_stadata_names(rain_ac)
            for n in range(len(names)-1):
                rain_ac[names[n]] /= rain_ac[names[-1]]
            rain_ac = meteva.base.in_member_list(rain_ac,names[:-1])
            rain_ac.attrs["valid_time"] = (times[-1] - times[0])/np.timedelta64(1, 'h')
            return rain_ac

        else:
            step = int(round(span/min_dhour))
            sta1["count_for_add"] = 1
            if not equal_weight:
                names = meteva.base.get_stadata_names(sta1)
                rain_ac = sta1.copy()
                for name in names:
                    rain_ac[name] *= 0.5

                for i in range(1,step):
                    rain1 = sta1.copy()
                    rain1["time"] = rain1["time"] + min_dtime * i
                    rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer",default=0)
                rain1 = sta1.copy()
                rain1["time"] = rain1["time"] + min_dtime * step
                for name in names:
                    rain1[name] *= 0.5
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer", default=0)

            else:
                rain_ac = None
                for i in range(step):
                    rain1 = sta1.copy()
                    rain1["time"] = rain1["time"] + min_dtime * i
                    rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer",default=0)

            names = meteva.base.get_stadata_names(rain_ac)
            for n in range(len(names)-1):
                rain_ac[names[n]] /= rain_ac[names[-1]]
            rain_ac = meteva.base.in_member_list(rain_ac,names[:-1])
            rain_ac = meteva.base.between_time_range(rain_ac, times[0], times[-1])  # 删除时效小于range的部分

            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva.base.in_time_list(rain_ac, new_times)

            rain_ac.attrs["valid_time"] = span
            return rain_ac

    elif used_coords ==["dtime"]:
        if span is None:
            print("if used_coords == [dtime], span must be int of float bigger than 0 ")

        dtimes = sta.loc[:, 'dtime'].values
        dtimes = list(set(dtimes))
        dtimes.sort()
        dtimes = np.array(dtimes)

        dhour_unit = dtimes[0]
        if dhour_unit == 0:
            dhour_unit = dtimes[1]
        step = int(round(span/dhour_unit))

        if equal_weight:
            sta1["count_for_add"] = 1
            names = meteva.base.get_stadata_names(sta1)
            rain_ac = sta1.copy()
            for i in range(1,step):
                rain1 = sta1.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1,how="outer",default=0)
        else:
            pass
            dtimes_delta = (dtimes[1:] - dtimes[:-1])
            names = meteva.base.get_stadata_names(sta1)
            sta1["weight_pre"] =0
            sta1["weight_later"] = 0
            for i in range(len(dtimes)-1):
                sta1.loc[sta1["dtime"] == dtimes[i+1],"weight_pre"] = dtimes_delta[i]
                sta1.loc[sta1["dtime"] == dtimes[i], "weight_later"] = dtimes_delta[i]

            sta2 = sta1.iloc[:,:-2]
            sta2.loc[:,"count_for_add"] = sta1.loc[:,"weight_pre"]
            rain_ac = sta2.copy()


            for n in range(len(names)):
                rain_ac.loc[:,names[n]] *= sta2.loc[:,"count_for_add"]

            sta2.loc[:,"count_for_add"] = sta1.loc[:,"weight_pre"] + sta1.loc[:,"weight_later"]
            for i in range(1,step-1):
                rain1 = sta2.copy()
                for n in range(len(names)):
                    rain1.loc[:,names[n]] *= sta2.loc[:,"count_for_add"]
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1,how="outer",default=0)

            sta2.loc[:,"count_for_add"] = sta1.loc[:,"weight_later"]
            rain1 = sta2.copy()
            for n in range(len(names)):
                rain1.loc[:,names[n]] *= sta2.loc[:,"count_for_add"]
            rain1["dtime"] = rain1["dtime"] + dhour_unit * step
            rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer", default=0)

        names = meteva.base.get_stadata_names(rain_ac)
        for n in range(len(names)-1):
            rain_ac[names[n]] /= rain_ac[names[-1]]

        rain_ac = meteva.base.in_member_list(rain_ac,names[:-1])
        rain_ac = meteva.base.in_dtime_list(rain_ac,dtimes[1:])  # 删除时效小于range的部分
        if not keep_all:
            dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
            new_dtimes = dtimes[dh % step == 0]
            rain_ac = meteva.base.in_dtime_list(rain_ac, new_dtimes)
        rain_ac.attrs["valid_time"] = span
        return rain_ac

def std_of_sta(sta,used_coords = ["member"]):
    sta_std = sta.loc[:,meteva.base.get_coord_names()]
    sta_data = sta[meteva.base.get_stadata_names(sta)]
    value = sta_data.values.astype(np.float32)
    std = np.std(value, axis=1)
    sta_std['std'] = std
    return sta_std

def var_of_sta(sta,used_coords = ["member"]):
    sta_var = sta.loc[:,meteva.base.get_coord_names()]
    sta_data = sta[meteva.base.get_stadata_names(sta)]
    value = sta_data.values
    var = np.var(value, axis=1)
    sta_var['var'] = var
    return sta_var

def max_of_sta(sta,used_coords = ["member"],span = None, contain_start = False,keep_all = True,ignore_missing = False):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    default = np.min(sta.iloc[:,6:].values)

    if ignore_missing:
        how = "outer"
    else:
        how = "inner"

    if used_coords  == ["member"]:
        sta_max = sta.loc[:,meteva.base.get_coord_names()]
        sta_data = sta[meteva.base.get_stadata_names(sta)]
        value = sta_data.values
        max1 = np.max(value, axis=1)
        sta_max['max'] = max1
        return sta_max
    elif used_coords == ["time"]:
        if span is None:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            rain_ac = meteva.base.in_time_list(sta,[times[0]])
            meteva.base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1,len(times)):
                rain01 = meteva.base.in_time_list(sta,times[i])
                meteva.base.set_stadata_coords(rain01,time=times[-1])
                rain_ac = meteva.base.max_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            times = np.array(times)
            dtimes = times[1:] - times[0:-1]
            min_dtime = np.min(dtimes)
            min_dhour = min_dtime / np.timedelta64(1, 'h')
            rain_ac = None
            step = int(round(span/min_dhour))
            if contain_start: step += 1
            for i in range(step):
                rain1 = sta.copy()
                rain1["time"] = rain1["time"] + min_dtime * i
                rain_ac = meteva.base.max_on_level_time_dtime_id(rain_ac, rain1, default=default)
            time0_add_span = times[0] + np.timedelta64(int(min_dhour * (step - 1)),'h')
            rain_ac = meteva.base.sele_by_para(rain_ac,time_range=[time0_add_span,times[-1]])
            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva.base.in_time_list(rain_ac, new_times)
            return rain_ac
    elif used_coords ==["dtime"]:
        if span is None:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            rain_ac = meteva.base.in_dtime_list(sta, [dtimes[0]])
            meteva.base.set_stadata_coords(rain_ac, dtime=dtimes[-1])
            for i in range(1, len(dtimes)):
                rain01 = meteva.base.in_dtime_list(sta, dtimes[i])
                meteva.base.set_stadata_coords(rain01, dtime=dtimes[-1])
                rain_ac = meteva.base.max_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            dtimes = np.array(dtimes)
            dhour_unit = dtimes[1] - dtimes[0]

            #if dhour_unit == 0:
            #    dhour_unit = dtimes[1]

            rain_ac = sta.copy()
            #print(span)
            #print(dhour_unit)
            step = int(round(span/dhour_unit))
            if contain_start: step += 1
            for i in range(1,step):
                rain1 = sta.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                # print(dhour_unit * i)
                rain_ac = meteva.base.max_on_level_time_dtime_id(rain_ac, rain1, default=default)


            begin_dtime = dtimes[0]+dhour_unit * (step - 1)
            rain_ac = meteva.base.between_dtime_range(rain_ac,begin_dtime,dtimes[-1])  # 删除时效小于range的部分
            dtimes =np.array(list(set(rain_ac.loc[:, "dtime"].values.tolist())))
            if not keep_all:
                dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
                new_dtimes = dtimes[dh % step == 0]
                rain_ac = meteva.base.in_dtime_list(rain_ac, new_dtimes)
            return rain_ac

def min_of_sta(sta,used_coords = ["member"],span = None,contain_start = False,keep_all = True,ignore_missing = False):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    default = np.max(sta.iloc[:,6:].values)
    if ignore_missing:
        how = "outer"
    else:
        how = "inner"
    if used_coords  == ["member"]:
        sta_min = sta.loc[:,meteva.base.get_coord_names()]
        sta_data = sta[meteva.base.get_stadata_names(sta)]
        value = sta_data.values
        min1 = np.min(value, axis=1)
        sta_min['min'] = min1
        return sta_min
    elif used_coords == ["time"]:
        if span is None:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            rain_ac = meteva.base.in_time_list(sta,[times[0]])
            meteva.base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1,len(times)):
                rain01 = meteva.base.in_time_list(sta,times[i])
                meteva.base.set_stadata_coords(rain01,time=times[-1])
                rain_ac = meteva.base.min_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            times = np.array(times)
            dtimes = times[1:] - times[0:-1]
            min_dtime = np.min(dtimes)
            min_dhour = min_dtime / np.timedelta64(1, 'h')
            rain_ac = None
            step = int(round(span/min_dhour))
            if contain_start:step += 1
            for i in range(step):
                rain1 = sta.copy()
                rain1["time"] = rain1["time"] + min_dtime * i
                rain_ac = meteva.base.min_on_level_time_dtime_id(rain_ac, rain1, default= default)
            time0_add_span = times[0] + np.timedelta64(int(min_dhour * (step - 1)),'h')
            rain_ac = meteva.base.sele_by_para(rain_ac,time_range=[time0_add_span,times[-1]])
            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva.base.in_time_list(rain_ac, new_times)
            return rain_ac
    elif used_coords ==["dtime"]:
        if span is None:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            rain_ac = meteva.base.in_dtime_list(sta, [dtimes[0]])
            meteva.base.set_stadata_coords(rain_ac, dtime=dtimes[-1])
            for i in range(1, len(dtimes)):
                rain01 = meteva.base.in_dtime_list(sta, dtimes[i])
                meteva.base.set_stadata_coords(rain01, dtime=dtimes[-1])
                rain_ac = meteva.base.min_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            dtimes = np.array(dtimes)
            dhour_unit = dtimes[1] - dtimes[0]

            #if dhour_unit == 0:
            #    dhour_unit = dtimes[1]

            rain_ac = sta.copy()
            #print(span)
            #print(dhour_unit)
            step = int(round(span/dhour_unit))
            if contain_start:step += 1

            for i in range(1,step):
                rain1 = sta.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                # print(dhour_unit * i)
                rain_ac = meteva.base.min_on_level_time_dtime_id(rain_ac, rain1,default=default)

            begin_dtime = dtimes[0]+dhour_unit * (step - 1)
            rain_ac = meteva.base.between_dtime_range(rain_ac,begin_dtime,dtimes[-1])  # 删除时效小于range的部分
            dtimes =np.array(list(set(rain_ac.loc[:, "dtime"].values.tolist())))
            if not keep_all:
                dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
                new_dtimes = dtimes[dh % step == 0]
                rain_ac = meteva.base.in_dtime_list(rain_ac, new_dtimes)
            return rain_ac

def sum_of_sta(sta,used_coords = ["member"],span = None,keep_all = True):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    if used_coords == ["member"]:
        sta_sum = sta.loc[:,meteva.base.get_coord_names()]
        sta_data = sta[meteva.base.get_stadata_names(sta)]
        value = sta_data.values
        min1 = np.sum(value, axis=1)
        sta_sum['min'] = min1
        return sta_sum

    elif used_coords == ["time"]:
        if span is None:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            rain_ac = meteva.base.in_time_list(sta,[times[0]])
            meteva.base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1,len(times)):
                rain01 = meteva.base.in_time_list(sta,times[i])
                meteva.base.set_stadata_coords(rain01,time=times[-1])
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain01, how="outer",default=0)
            return rain_ac
        else:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            times = np.array(times)
            dtimes = times[1:] - times[0:-1]
            min_dtime = np.min(dtimes)
            min_dhour = min_dtime / np.timedelta64(1, 'h')
            rain_ac = None
            step = int(round(span/min_dhour))
            for i in range(step):
                rain1 = sta.copy()
                rain1["time"] = rain1["time"] + min_dtime * i
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1, how="inner")

            time0_add_span = times[0] + np.timedelta64(int(min_dhour * (step - 1)),'h')
            rain_ac = meteva.base.sele_by_para(rain_ac,time_range=[time0_add_span,times[-1]])
            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva.base.in_time_list(rain_ac, new_times)
            return rain_ac
    elif used_coords ==["dtime"]:
        if span is None:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            rain_ac = meteva.base.in_dtime_list(sta, [dtimes[0]])
            meteva.base.set_stadata_coords(rain_ac, dtime=dtimes[-1])
            for i in range(1, len(dtimes)):
                rain01 = meteva.base.in_dtime_list(sta, dtimes[i])
                meteva.base.set_stadata_coords(rain01, dtime=dtimes[-1])
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain01, how="outer",default=0)

                # rain_ac0 = rain_ac.copy()
                # a = rain_ac0.duplicated(subset=["level","time","dtime","id"])
                # for ii in range(len(a)):
                #     if a[ii]:
                #         ids = rain_ac["id"].values[ii]
                #         sta4 = meteva.base.sele_by_para(rain_ac,id = ids)
                #         print(sta4)
                #         print()

                #rain_ac.drop_duplicates(subset=["level","time","dtime","id"],inplace=True)
            return rain_ac
        else:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            dtimes = np.array(dtimes)
            dhour_unit = dtimes[1] - dtimes[0]

            #if dhour_unit == 0:
            #    dhour_unit = dtimes[1]

            rain_ac = sta.copy()
            #print(span)
            #print(dhour_unit)
            step = int(round(span/dhour_unit))
            #print(step)
            for i in range(1,step):
                rain1 = sta.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                # print(dhour_unit * i)
                rain_ac = meteva.base.add_on_level_time_dtime_id(rain_ac, rain1, default=0)

            begin_dtime = dtimes[0]+dhour_unit * (step - 1)
            rain_ac = meteva.base.between_dtime_range(rain_ac,begin_dtime,dtimes[-1])  # 删除时效小于range的部分
            dtimes =set(rain_ac.loc[:, "dtime"].values.tolist())
            dtimes = list(dtimes)
            dtimes.sort()
            dtimes = np.array(dtimes)
            if not keep_all:
                dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
                new_dtimes = dtimes[dh % step == 0]
                rain_ac = meteva.base.in_dtime_list(rain_ac, new_dtimes)
            return rain_ac

#获取网格数据的平均值
def mean_of_grd(grd,used_coords = ["member"],span = None):

    if used_coords==["member"]:
        grid0 = meteva.base.basicdata.get_grid_of_data(grd)
        grid1 = meteva.base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["mean"])
        dat = np.squeeze(grd.values)
        if len(dat.shape) > 2:
            dat = np.mean(dat,axis = 0)
        grd1 = meteva.base.basicdata.grid_data(grid1,dat)
        return grd1
    elif used_coords == "dtime" or used_coords == ["dtime"]:
        if span == None:
            grid0 = meteva.base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = [dtimes[-1]]
            grid1 = meteva.base.grid(grid0.glon, grid0.glat, grid0.gtime, dtimes_sum, grid0.levels, grid0.members)
            grd_mean = meteva.base.grid_data(grid1)
            grd_mean.values[:, :, :, 0, :, :] = np.mean(grd.values[:, :, :, :, :, :], axis=3)
            return grd_mean
        else:
            grid0 = meteva.base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = dtimes[dtimes >= span]
            grid1 = meteva.base.grid(grid0.glon, grid0.glat, grid0.gtime, dtimes_sum, grid0.levels, grid0.members)
            grd_mean = meteva.base.grid_data(grid1)
            for i in range(len(dtimes_sum)):
                dtime_e = dtimes_sum[i]
                dtime_s = dtimes_sum[i] - span
                index = np.where((dtimes > dtime_s) & (dtimes <= dtime_e))[0]

                grd_mean.values[:, :, :, i, :, :] = np.mean(grd.values[:, :, :, index, :, :], axis=3)
            return grd_mean
    elif used_coords =="time" or used_coords ==["time"]:
        if span ==None:
            grid0 = meteva.base.get_grid_of_data(grd)
            grid1 = meteva.base.grid(grid0.glon, grid0.glat, [grid0.gtime[1]], grid0.dtimes, grid0.levels,
                                     grid0.members)
            grd_mean = meteva.base.grid_data(grid1)

            grd_mean.values[:, :, 0, :, :, :] = np.mean(grd.values[:, :, :, :, :, :], axis=2)
            return grd_mean
        else:
            grid0 = meteva.base.get_grid_of_data(grd)
            grd_mean = meteva.base.grid_data(grid0)
            values = grd.values.copy()
            step = grid0.dtime_int
            half = int(span/step/2)
            kernal = np.ones(half * 2 +1)
            kernal[0] = 0.5
            kernal[-1] = 0.5
            shape = values.shape
            ntime = values.shape[2]
            for i in range(ntime):
                sum_v = np.zeros((shape[0],shape[1],shape[3],shape[4],shape[5]))
                sum_q = 0
                for k in range(-half,half+1,1):
                    i_k = i + k
                    if i_k>=0 and i_k< values.shape[2]:
                        q = kernal[k + half]
                        sum_v += grd.values[:, :, i_k, :, :, :] * q
                        sum_q+= q
                sum_v /= sum_q
                grd_mean.values[:, :, i, :, :, :] = sum_v

            return grd_mean



#获取网格数据的方差
def var_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    grid1 = meteva.base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["var"])
    dat = np.squeeze(grd.values)
    if len(dat.shape) > 2:
        dat = np.var(dat,axis = 0)
    grd1 = meteva.base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的标准差
def std_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    grid1 = meteva.base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["std"])
    dat = np.squeeze(grd.values)
    if len(dat.shape) > 2:
        dat = np.std(dat,axis = 0)
    grd1 = meteva.base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的最小值
def min_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    grid1 = meteva.base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["min"])
    dat = np.squeeze(grd.values)
    if len(dat.shape)>2:
        dat = np.min(dat,axis = 0)
    grd1 = meteva.base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的最大值
def max_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    grid1 = meteva.base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["max"])
    dat = np.squeeze(grd.values)
    if len(dat.shape)>2:
        dat = np.max(dat,axis = 0)
    grd1 = meteva.base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的求和
def sum_of_grd(grd,used_coords = ["member"],span = None):
    if used_coords ==["member"]:
        grid0 = meteva.base.basicdata.get_grid_of_data(grd)
        grid1 = meteva.base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["max"])
        dat = np.squeeze(grd.values)
        if len(dat.shape) > 2:
            dat = np.sum(dat,axis = 0)
        grd1 = meteva.base.basicdata.grid_data(grid1,dat)
        return grd1
    elif used_coords == "dtime" or used_coords ==["dtime"]:
        if span == None:
            grid0 = meteva.base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = [dtimes[-1]]
            grid1 = meteva.base.grid(grid0.glon, grid0.glat, grid0.gtime, dtimes_sum, grid0.levels, grid0.members)
            grd_sum = meteva.base.grid_data(grid1)
            grd_sum.values[:, :, :, 0, :, :] = np.sum(grd.values[:, :, :, :, :, :], axis=3)
            return grd_sum
        else:
            grid0 = meteva.base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = dtimes[dtimes>=span]
            grid1 = meteva.base.grid(grid0.glon,grid0.glat,grid0.gtime,dtimes_sum,grid0.levels,grid0.members)
            grd_sum = meteva.base.grid_data(grid1)
            for i in range(len(dtimes_sum)):
                dtime_e = dtimes_sum[i]
                dtime_s = dtimes_sum[i] - span
                index = np.where((dtimes > dtime_s) & (dtimes <= dtime_e))[0]

                grd_sum.values[:,:,:,i,:,:] = np.sum(grd.values[:,:,:,index,:,:],axis=3)
            return grd_sum
    elif used_coords =="time" or used_coords ==["time"]:
        if span ==None:
            grid0 = meteva.base.get_grid_of_data(grd)
            grid1 = meteva.base.grid(grid0.glon, grid0.glat, [grid0.gtime[1]], grid0.dtimes, grid0.levels, grid0.members)
            grd_sum = meteva.base.grid_data(grid1)
            grd_sum.values[:, :, 0, :, :, :] = np.sum(grd.values[:, :, :, :, :, :], axis=2)
            return grd_sum
        else:
            grid0 = meteva.base.get_grid_of_data(grd)
            grd_sum = meteva.base.grid_data(grid0)
            values = grd.values.copy()
            step = grid0.dtime_int
            count = int(span/step)

            shape = values.shape
            ntime = values.shape[2]
            for i in range(ntime):
                sum_v = np.zeros((shape[0],shape[1],shape[3],shape[4],shape[5]))
                for k in range(-count+1,1):
                    i_k = i + k
                    if i_k <0:continue
                    sum_v += grd.values[:, :, i_k, :, :, :]

                grd_sum.values[:, :, i, :, :, :] = sum_v

            grd_sum = grd_sum.isel(time = slice(count-1,ntime))
            return grd_sum



def time_ceilling(sta,step = 1, time_unit = "h",begin_hour= 8):
    '''
    将不规则时间观测数据累计到固定步长的整点时刻
    :param sta: 站点数据，例如原始的闪电观测数据
    :param step:  累计的步长
    :param time_unit: 累计的时间单位，可选项包括 “H"和”M"，分别代表小时和分钟。
    :param begin_hour: 时间类型，当累计步长超过1小时,例如3小时，起步累计的时间是从08时还是00时，通常对北京时数据来说以08时起步，世界时则以00时起步
    :return:
    '''
    sta1 = sta.copy()
    time0 = datetime.datetime(2000,1,1,begin_hour,0)
    if time_unit.lower() == "h":
        step *= 3600
    else:
        step *= 60
    delta = ((sta["time"] - time0)/np.timedelta64(1,"s")).values
    delta =np.ceil(delta/step) * step
    sta1["time"] = time0
    sta1["time"] += delta * np.timedelta64(1, "s")
    return sta1

def loc_of_max(sta,used_coords = ["dtime"],ignore_missing = False):
    '''
    返回站点数据在某些维度上最大值的坐标
    :param sta:
    :param used_coords:
    :param span:
    :param keep_all:
    :param ignore_missing:
    :return:
    '''
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    default = np.min(sta.iloc[:,6:].values)
    if ignore_missing:
        how = "outer"
    else:
        how = "inner"
    if used_coords  == ["dtime"]:
        dtimes = sta.loc[:, 'dtime'].values
        dtimes = list(set(dtimes))
        dtimes.sort()
        dtime_array = np.array(dtimes)
        data_names = meteva.base.get_stadata_names(sta)
        max_loc = None
        for data_name in data_names:
            sta1 = meteva.base.in_member_list(sta,[data_name])

            sta1_0 = meteva.base.in_dtime_list(sta1,[dtimes[0]])
            meteva.base.set_stadata_coords(sta1_0, dtime=0)
            for i in range(1, len(dtimes)):
                sta1_1 = meteva.base.in_dtime_list(sta1, dtimes[i])
                meteva.base.set_stadata_coords(sta1_1, dtime=0)
                meteva.base.set_stadata_names(sta1_1,data_name_list=[data_name+"_"+str(i)])
                sta1_0 = pd.merge(sta1_0, sta1_1, on=["level", "time", "dtime", "id","lon","lat"], how=how)

            sta1_0.fillna(default, inplace=True)

            all_values = sta1_0.iloc[:, 6:].values
            max_index = np.argmax(all_values,axis=1)
            max_dtime = dtime_array[max_index]
            sta1_dtime = meteva.base.in_member_list(sta1_0,[data_name])
            sta1_dtime.iloc[:,-1] = max_dtime[:]
            max_loc = meteva.base.combine_on_level_time_dtime_id(max_loc,sta1_dtime)

        return max_loc

def loc_of_min(sta,used_coords = ["dtime"],ignore_missing = False):
    sta1 = sta.copy()
    sta1.iloc[:,6:] *= -1
    return loc_of_max(sta1,used_coords=used_coords,ignore_missing = ignore_missing)



# 采用以下形式将所有.py 模块导入到mode 目录下
from . import angles_psp
from . import as_psp
from . import lengths_psp
from . import midpoints_psp
from . import bearing
from . import centmatch
from . import censqdelta
from . import data_pre
from . import deltametric
from . import deltamm
from . import deltammSqCen
from . import distmap
from . import feature_axis
from . import feature_comps
from . import feature_finder
from . import feature_match_analyzer
from . import feature_props
from . import feature_table
from . import interester
from . import intersect
from . import loc_list_setup
from . import locperf
from . import make_spatialVx
from . import merge_force
from . import minboundmatch
from . import sma
from . import utils
from . import plot
from . import operater
from . import load
from . import tran_to_dataframe
from . import consistent


# 采用如下形式将5个用户必要的函数导入到mode模块下
from .make_spatialVx import make_spatialVx
from .feature_finder import feature_finder,feature_finder_single,feature_finder_and_merge
from .centmatch import centmatch
from .merge_force import merge_force,merge
from .deltamm import deltamm
from .minboundmatch import minboundmatch
from .feature_table import feature_table
from .feature_axis import feature_axis,caculate_feature_axis
from .feature_props import feature_props
from .feature_comps import feature_comps
from .feature_match_analyzer import  feature_merged_analyzer,get_summary
from .plot import plot_value,plot_label,plot_value_and_label,plot_interest,plot_feature,plot_value_list,plot_label_list,plot_value_and_label_list
from .utils import get_attributes_for_feat,remove_key_from_list
from .interester import interester
from .operater import operate
from .load import load_feature_summary,load_feature_summary_list
from .tran_to_dataframe import features_list_to_df
from .consistent import combined_near_labels,unimatch,unimerge



import numpy as np
import meteva
import math
import sys
import copy
from . import data_pre
from .feature_props import feature_props
from meteva.method.space.mode.feature_finder import get_disk_kernel,propCounts
from scipy.ndimage import convolve
from scipy.spatial import cKDTree

def rotate_and_pull(pts,angle,gama):
    '''
    根据目标的主轴方向和形态，确定变形后坐标
    :param pts:
    :param angle:
    :param gama:
    :return:
    '''
    x = (math.cos(angle) * pts[:, 0] + math.sin(angle) * pts[:, 1])*gama
    y = -math.sin(angle) * pts[:, 0] + math.cos(angle) * pts[:, 1]
    pts_new = np.array([x,y]).T
    return  pts_new


def caculate_near_rate(pts1,pts2,near_dis,add_min_d = False):
    axis_feature1 = meteva.method.mode.caculate_feature_axis(pts1)
    lengths_max = axis_feature1["lengths"]["MajorAxis"]
    lengths_min = axis_feature1["lengths"]["MinorAxis"]
    gama = max(max(lengths_min,near_dis)/max(lengths_max,near_dis),0.5) #计算主轴方向距离压缩比例
    angle= axis_feature1["OrientationAngle"]["MajorAxis"] * math.pi/180
    pts1_rp = rotate_and_pull(pts1,angle,gama)
    pts2_rp = rotate_and_pull(pts2,angle,gama)
    tree = cKDTree(pts1_rp)
    d,_ = tree.query(pts2_rp, k=1)
    if add_min_d:
        min_d = np.min(d)
        rate = len(d[d<near_dis])/len(d) + 0.001/min_d
    else:
        rate = len(d[d < near_dis]) / len(d)
    return rate

def caculate_cover_rate(pts1,pts2,near_dis):

    '''
    计算观测目标和预报目标之间的覆盖度
    :param pts1:
    :param pts2:
    :param near_dis:
    :return:
    '''
    near_rate1 = caculate_near_rate(pts2,pts1,near_dis)
    count1 =near_rate1 * pts1.shape[0]

    near_rate2 = caculate_near_rate(pts1,pts2,near_dis)
    count2 =near_rate2 * pts2.shape[0]

    rate = (count1 + count2) /(pts1.shape[0] +pts2.shape[0])
    rate -= 0.01 / pts2.shape[0]
    return rate

def combined_near_labels(labelsfeature,near_dis,near_rate):
    labels =[]
    label_count0 = labelsfeature["label_count"]
    labelsfeature_new = copy.deepcopy(labelsfeature)
    if label_count0 ==0:
        return labelsfeature_new
    for i in range(label_count0):
        labelsfeature_new.pop(i+1)

    #print(labelsfeature.keys())
    for i in range(label_count0):
        label_ps1 = np.array([labelsfeature[i+1][1],labelsfeature[i+1][0]]).T
        labels.append(label_ps1)

    max_rate = 1
    while max_rate >= near_rate:
        nlabels = len(labels)
        rate_array = np.zeros((nlabels,nlabels))
        for i in range(0,nlabels):
            for j in range(nlabels):
                if j==i:continue
                rate_array[i,j] = caculate_near_rate(labels[i],labels[j],near_dis,add_min_d=True)
        max_rate = np.max(rate_array)
        if max_rate > near_rate:
            index = np.where(rate_array == max_rate)
            i = index[0][0]
            j = index[1][0]
            labels[i] = np.append(labels[i],labels[j],axis=0)
            del labels[j]




    #print(len(labels))
    area_list = []
    for i in range(len(labels)):
        area_list.append(labels[i].shape[0])
    area_array = np.array(area_list)
    #print(area_list)

    sort_index = len(labels)-1 - np.argsort(np.argsort(area_array))

    for i in range(len(labels)):
        j = sort_index[i]
        labelsfeature_new[j+1] = (labels[i][:,1],labels[i][:,0])
    area_array.sort()

    labelsfeature_new["area"] =area_array[::-1]
    labelsfeature_new["label_count"] = len(labels)
    #print(labelsfeature)
    return labelsfeature_new


def unimatch(look_ob,look_fo,cover_dis,cover_rate):
    out = copy.deepcopy(look_fo)

    grid0= look_ob["grid"]
    near_dis = cover_dis/111/grid0.dlon
    label_count_ob = look_ob["grd_features"]["label_count"]
    label_count_fo = look_fo["grd_features"]["label_count"]
    if label_count_ob >0 and label_count_fo >0:
        labels_ob = []
        for i in range(label_count_ob):
            label_ps1 = np.array([look_ob["grd_features"][i+1][1],look_ob["grd_features"][i+1][0]]).T
            labels_ob.append(label_ps1)

        labels_fo = []

        for i in range(label_count_fo):
            label_ps2 = np.array([look_fo["grd_features"][i+1][1],look_fo["grd_features"][i+1][0]]).T
            labels_fo.append(label_ps2)

        max_rate = 1

        combined_fo_dict = {}
        used_ob = np.ones(label_count_ob)
        used_fo = np.ones(label_count_fo)
        while max_rate >= cover_rate:
            max_rate = 0
            max_i = -1
            max_j_list = None
            max_pts_fo = None
            for i in range(label_count_ob):
                if used_ob[i] ==0:continue
                rate_1row = np.zeros(label_count_fo)
                for j in range(label_count_fo):
                    if used_fo[j] ==0:continue
                    rate_1row[j] = caculate_cover_rate(labels_ob[i],labels_fo[j],near_dis)
                index = np.argsort(-rate_1row)
                pts_fo = labels_fo[index[0]]
                for j in range(0,label_count_fo):
                    if rate_1row[index[j]] > 0:
                        if j > 0:
                            pts_fo =  np.append(pts_fo,labels_fo[index[j]],axis=0)
                            rate_c = caculate_cover_rate(labels_ob[i],pts_fo,near_dis)
                            if rate_c > max_rate:
                                max_rate = rate_c
                                max_i = i
                                max_j_list = index[0:j+1]
                                max_pts_fo = pts_fo
                        else:
                            rate_c = rate_1row[index[j]]
                            if rate_c> max_rate:
                                max_rate = rate_c
                                max_i = i
                                max_j_list = [index[0]]
                                max_pts_fo = pts_fo

                    else:
                        break


            if max_rate > cover_rate:
                combined_fo_dict[max_i+1] = (max_pts_fo[:,1],max_pts_fo[:,0])
                used_fo[max_j_list] = 0
                used_ob[max_i] =0


        id_list = list(combined_fo_dict.keys())
        nmatch =0
        if len(id_list)>0:
            nmatch = max(combined_fo_dict.keys())
        #print(combined_fo_dict)

        kk = label_count_ob
        for k in range(len(labels_fo)):
            if used_fo[k] > 0:
                kk += 1
                combined_fo_dict[kk] = (labels_fo[k][:, 1], labels_fo[k][:, 0])
                id_list.append(kk)

        grd_fo_labeled = look_fo["grd"].copy()
        grd_fo_labeled.attrs["var_name"] = "目标编号"
        label_value = np.zeros((grid0.nlat, grid0.nlon))
        for key in combined_fo_dict.keys():
            label = combined_fo_dict[key]
            label_value[label] = key
        grd_fo_labeled.values[:] = label_value[:]
        combined_fo_dict["label_count"] = kk
        for key in look_fo["grd_features"].keys():
            if key not in combined_fo_dict.keys():
                if isinstance(key,str):
                    combined_fo_dict[key] = copy.deepcopy(look_fo["grd_features"][key])
        out["grd_features"]= combined_fo_dict
        out["grd_label"]= grd_fo_labeled
        out["match_count"] = nmatch
        out["max_label"] = kk
        out["id_list"] =id_list
    out["match_type"] = "unimatch"


    return out


def unimerge(look_ob,look_fo):

    label_list_ob = look_ob["id_list"]
    label_list_fo = look_fo["id_list"]
    label_list_matched = list(set(label_list_ob) & set(label_list_fo))
    label_list_all = list(set(label_list_ob) | set(label_list_fo))
    vxunmatched = copy.deepcopy(label_list_ob)
    fcunmatched = copy.deepcopy(label_list_fo)
    for id in label_list_matched:
        vxunmatched.remove(id)
        fcunmatched.remove(id)

    if len(label_list_all)>0:
        max_label = np.max(np.array(label_list_all))
    else:
        max_label = 0

    out = {'grd_ob': look_ob["grd"],
           "grd_ob_smooth": look_ob["grd_smooth"],
           "grd_ob_features": look_ob["grd_features"],
           "grd_ob_label": look_ob["grd_label"],
           'grd_fo': look_fo["grd"],
           "grd_fo_smooth": look_fo["grd_smooth"],
           "grd_fo_features": look_fo["grd_features"],
           "grd_fo_label": look_fo["grd_label"],
           "grid":look_ob["grid"],
           "match_count":len(label_list_matched),
           "max_label" : max_label,
           "label_list_ob":label_list_ob,
           "label_list_fo":label_list_fo,
           "label_list_matched":label_list_matched,
           "label_list_all":label_list_all,
           "unmatched":{'ob': vxunmatched, 'fo': fcunmatched},
           "matches":np.array([label_list_matched,label_list_matched]).T,
           "match_type":look_fo["match_type"]}

    return out